#!/usr/bin/python

# Copyright (c) 2008 Stanford University.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# - Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the
#   distribution.
# - Neither the name of the Stanford University nor the names of
#   its contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL STANFORD
# UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# @author Kannan Srinivasan <srikank@stanford.edu>
# @author Mayank Jain <mayjain@stanford.edu>
# @author Jonathan Chu <jchu3@stanford.edu>
# Created: 11/05/2008
import sys, os
from datetime import datetime
import subprocess
DB_TABLE_NAME=sys.argv[1]
ETX_THRESHOLD=3.0
ASYMM_THRESHOLD=0.4
logdir="../log/%s" % DB_TABLE_NAME
nodeconfig="nodes"
configFile=DB_TABLE_NAME.partition("_")[0]

os.chdir("../experimentfiles")

#----- File to output table
mytableFile="%s/tablefile" % logdir
tableFile = open(mytableFile, "w")
tableFile.close()

#----- The node file to read
mynodesFile="%s/%s.txt" % (logdir, nodeconfig)

#----- The config file to read
myswatFile="%s/%s.config" % (logdir, configFile)

config_data = {}
swatFile = open(myswatFile, "r")
for line in swatFile:
    parts = line.split()
    config_data[parts[0]] = parts[1]
swatFile.close()
#fieldname=( "experiment_name:" "interface:" "channel:" "power_level:" "mote:" "num_nodes:" "num_packets:" "ipi:" "csma:" "transmission:" "ack:" "max_retries:" "noise_sample:" "num_samples:" "sample_ipi:" )

NUMNODES=int(config_data["num_nodes:"])
PACKETS=int(config_data["num_packets:"])
RAWIPI=int(config_data["ipi:"])
txtype=config_data["transmission:"]

if txtype == "unicast":
    ack=config_data["ack:"]
    if ack == "yes":
        ACK=True
    elif ack == "no":
        ACK=False
elif txtype == "broadcast":
    ACK=False

noisesample=config_data["noise_sample:"]

if noisesample == "yes":
    NOISE=True
elif noisesample == "no":
    NOISE=False

os.chdir("../metrics")
sys.path.append(os.getcwd())
from prrcalc import calculateall
from prrdistr import plotprrdistr
from prrvsrssi import plotprrvsrssi
from prrvscci import plotprrvscci
from rssivscci import plotrssivscci
from linkasymm import plot_linkasymm
from noisedistr import plotnoisedistr
from snrasymm import plotsnrasymm
from prrvsarr import plot_prrvsarr
from beta import plotbimodality
from etxasymm import plot_etxasymm
from topology import generate_topology

try:
    os.mkdir("../reports/%s" % DB_TABLE_NAME)
except OSError:
    pass
try:
    os.mkdir("../reports/%s/plots" % DB_TABLE_NAME)
except OSError:
    pass

totplots=8

if NOISE:
    totplots += 2
if NUMNODES > 2:
    totplots += 1
if ACK:
    totplots += 2
else:
    totplots += 1

plotsdone=1

myStatusFile="%s/status.txt" % logdir
statusFile = open(myStatusFile, "r+")
seek_after = len(statusFile.readline()) + len(statusFile.readline())

def print_message(str):
    print "%s: %s" % (str, datetime.now().ctime())

def plot_done(str):
    global plotsdone
    statusFile.seek(seek_after)
    statusFile.write("%s\n" % str)
    statusFile.flush()
    plotsdone += 1

def ignore_errors(func, *args):
    retval = None
    try:
        retval = func(*args)
    except Exception, e:
        print e.message
    return retval

increment = 25
print_message("calculating prr and basic metrics")
[prrarr, avgrssi, stdrssi, avgcci, stdcci, asymm_links] = calculateall(DB_TABLE_NAME, PACKETS, increment)

print_message("calculating prr distribution")
ignore_errors(plotprrdistr, DB_TABLE_NAME, PACKETS, RAWIPI, NUMNODES, prrarr, avgrssi, stdrssi, avgcci, stdcci)
plot_done("metrics: inprogress %d %d" % (plotsdone, totplots))

print_message("calculating prr vs rssi")
ignore_errors(plotprrvsrssi, DB_TABLE_NAME, prrarr, avgrssi, stdrssi, avgcci, stdcci)
plot_done("metrics: inprogress %d %d" % (plotsdone, totplots))

#the following graphs are only relevant in 802.15.4
if config_data["interface:"]  == "802.15.4":
    print_message("calculating prr vs cci")
    ignore_errors(plotprrvscci, DB_TABLE_NAME, prrarr, avgrssi, stdrssi, avgcci, stdcci)
    plot_done("metrics: inprogress %d %d" % (plotsdone, totplots))

    print_message("calculating rssi vs cci")
    ignore_errors (plotrssivscci, DB_TABLE_NAME, prrarr, avgrssi, stdrssi, avgcci, stdcci)
    plot_done("metrics: inprogress %d %d" % (plotsdone, totplots))

print_message("computing average degree")
subprocess.call(["python", "avgdegree.py", DB_TABLE_NAME])

for i in range(len(asymm_links)):
    current = i * increment
    next = current + increment
    print_message("computing link asymmetry %d-%d" % (current, next))
    ignore_errors(plot_linkasymm, DB_TABLE_NAME, ASYMM_THRESHOLD, current, next, asymm_links[i])
    plot_done("metrics: inprogress %d %d" % (plotsdone, totplots))

if NOISE:
    print_message("computing noise distribution")
    ignore_errors(plotnoisedistr, DB_TABLE_NAME)
    plot_done("metrics: inprogress %d %d" % (plotsdone, totplots))

    print_message("computing snr asymmetry")
    ignore_errors(plotsnrasymm, DB_TABLE_NAME, RAWIPI)
    plot_done("metrics: inprogress %d %d" % (plotsdone, totplots))

if not ACK:
    print_message("computing beta")
    ignore_errors(plotbimodality, DB_TABLE_NAME, PACKETS, RAWIPI)
    plot_done("metrics: inprogress %d %d" % (plotsdone, totplots))

else:
    print_message("computing prr vs arr")
    arr = ignore_errors(plot_prrvsarr, DB_TABLE_NAME, prrarr)
    tableFile = open(mytableFile, "a")
    tableFile.write("beta: n/a n/a\n")
    tableFile.close()
    plot_done("metrics: inprogress %d %d" % (plotsdone, totplots))

    print_message("computing etx asymmetry")
    ignore_errors(plot_etxasymm, DB_TABLE_NAME, ETX_THRESHOLD, prrarr, arr)
    plot_done("metrics: inprogress %d %d" % (plotsdone, totplots))

#only calculate rho metric if more than two nodes are used
if NUMNODES > 2:
    print_message("computing rho")
    subprocess.call(["./alltxscript-xcorr", DB_TABLE_NAME, str(PACKETS)])
    plot_done("metrics: inprogress %d %d" % (plotsdone, totplots))

#generate topology
ignore_errors(generate_topology, DB_TABLE_NAME, prrarr)

plot_done("metrics: success")
statusFile.truncate()
statusFile.close()
