#!/usr/bin/env python

# Copyright (c) 2008 Stanford University.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# - Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the
#   distribution.
# - Neither the name of the Stanford University nor the names of
#   its contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL STANFORD
# UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# @author Edward Kim <edskim@stanford.edu>
# @author Kannan Srinivasan <srikank@stanford.edu>
# @author Jonathan Chu <jchu3@cs.stanford.edu>
# Created: 11/05/2008

import sys, random 
import tos2 as tos
import thread
import string
from threading import Thread, Lock
import db_functions
from time import sleep
from pylab import zeros, ones, int8

#contains the id-total packets received pairs (for the terminate() function)
totals = []
done = False
mutex = Lock()

#function that runs as a thread and updates the progress for the visualization
def record_progress():
    global counters, mutex, done
    # can't actually selectively remove lines from data.txt (at a new round, put all the nodes in the file,
    # but still in the current round, leave them  out) due to race conditions
    mutex.acquire()
    send_data = open("../ui/report_progress/data.txt", "r+")
    count_bytes = 0
    # skip over irrelevant lines (slow)
    for i in range(3):
        count_bytes += len(send_data.readline())
    for i in range(nomotes):
        line = send_data.readline()
        count_bytes += len(line)
        totals.append(line)
    # periodically update file
    while True:
        sleep(3)
        for index, counter in enumerate(counters):
            if counter:
                send_data.write("%s\t%d\n" % (id_array[index], counter))
        send_data.flush()
        if done:
            sleep(3)
            mutex.release()
            send_data.close()
            break

        #skip over irrelevant lines (fast)
        send_data.seek(count_bytes)

#things to do before terminating this round
def terminate():
    global linkfile, mutex, done
    done = True
    # wait for the thread recording the progress to stop
    mutex.acquire()
    send_data = open("../ui/report_progress/data.txt", "r+")
    #skip over irrelevant lines
    for i in range(3):
        send_data.readline()
    #update total packets received for each id
    for i in range(nomotes):
        count = int(totals[i].split()[1]) + counters[i]
        send_data.write("%s\t%d\n" % (id_array[i], count))
    send_data.truncate()
    send_data.close()
    linkfile.close()
    print "[ python tos-dump.py end ]"

#################################################################################
#										#
#				Function Blocks					#
#										#
#################################################################################

def dump(port, thisMoteID, index):

#Set Channel of this mote using parsed user specifications
#########################################################################
    def setChannel(): 							#
        chlpkt = tos.Command(None)					#
        chlpkt.channel = channel 					#
        chlpkt.number = 0 						#has to be 0 for channel setting
        chlpkt.power = power 						#
        am.write(chlpkt, 1) 						#
#########################################################################


#Send Command to all the motes to initiate noise sampling
#########################################################################
    def setNoise(): 							#
            npkt = tos.Command(None) 					#
            npkt.channel = 0 						#has to be 0 for starting noise sampling
            npkt.number = nnumber					#no. of noise samples
            npkt.ipi = nipi						#no. of noise samples per second
            am.write(npkt, 1) 						#
#########################################################################


#Initiate packet transmission once all receivers have set channel
#########################################################################
    def transmit():							#
            if destID == broadcastID:					#
                while moteRdyFlag.sum() < nomotes:			#
                    sleep(1.0/100.0)				#
            else:							#
                while moteRdyFlag[dest_position] != 1:	                #
                    sleep(1.0/100.0)				#
								#########
									#
									#
            pkt = tos.Command(None)					#
            pkt.ipi = ipi 						#
            pkt.number = number 					#
            pkt.channel = channel 					#
            pkt.dest = destID 						#0xFFFF for broadcast packets
            pkt.power = power						#
            pkt.acks = acks						#
            pkt.csma = csma						#
            am.write(pkt,1)						#
#########################################################################


#This block listens to the serial connection established with the mote
#It will wait for a packet and determine it's type
#If no packet is received for RxTIMEOUT seconds, loop will break
#The while loop will break when no more packets are expected from the mote
#################################################################################
    def listenSerialRxTx():						#
        global counters
        ackbit = 0								#
        retrans = 0								#
        packnum = 0								#
        npacknum = 0								#
        curr_seq = -1
										#
        if port == source_port :						#
            if acks == 0 or destID == broadcastID:				#If transmitter packet source will not expect BackToPC packets (ie Broadcast  or Unicast with no ack)
                packnum = number						#
            if number == 0 :							#If transmitter not to send any packets, mark transmission complete flag
                transCmplt[0] = 0						#
										#
        while True:								#Will break while loop after timeout max or shortly after last packet received
            timeout1 = (number - packnum)*ipi/1000.0
            timeout2 = (nnumber - npacknum)*nipi / 1000.0
            if timeout1 < timeout2:
                RxTIMEOUT = timeout2
            else:
                RxTIMEOUT = timeout1
            RxTIMEOUT += 3.0
            a = am.read(timeout=RxTIMEOUT)					#
            if a:
                #b = tos.ActiveMessage(a)					#
                if (a.type == 3):  #b.type == 3):				#
                    p = tos.BackToPC(a.data)					#
			        						#
                    ackflag = int(p.ackFlag)					#
										#
                    if p.lqi != 255:						#Check if TxRx Packet
                        packnum = p.packno					#Store packet number
                        if port == source_port :				#If transmitter, extract number retries from ack field
                            if ackflag > 1:					#
                                ackbit = 1					#
                                retrans = ackflag -1				#
                            elif ackflag == 1:					#
                                ackbit = ackflag				#
                                retrans = 0					#
                            ackData.append("%d %d %d %d %d %d %d\n" % (p.receiver, sourceID, p.packno, int8(p.rssi)+dbmOffset, p.lqi, ackbit, retrans))
                        else :
                            pktData.append("%d %d %d %d %d %d %d\n" % (sourceID, p.receiver, p.packno, int8(p.rssi)+dbmOffset, p.lqi, ackbit, retrans))
                            #suppress retransmissions
                            if curr_seq != packnum:
                                counters[index] += 1
                                curr_seq = packnum

                    else:							#
                        ackbit = 0						#If noise sample is received, ack and retrans fields set to 0
                        retrans = 0						#
                        npacknum = p.packno					#
                        noiseData.append("%d %d %d %d %d %d %d\n" % (sourceID, p.receiver, p.packno, int8(p.rssi)+dbmOffset, p.lqi, ackbit, retrans))
										#
										#
                    if packnum == number and npacknum == nnumber:		#If last packet and last noise sample have been received
                        if port != source_port:					#If mote is receiver, return		
                            return						#
                        elif transCmplt[0] == 0:				#If mote is transmitter and transmission is complete, return
                            return						#
										#
										#
                elif (a.type == 2):  #b.type == 2):				#if type Pack received from transmitter exit system if no other packets are expected
                    transCmplt[0] = 0						#Let program know that last packet has been sent
                    if packnum == number and npacknum == nnumber:  		#If no BackToPC packets or noise samples expected, exit system			
                        return							#
										#
            else:								#Timed out
                if port == source_port:						#
                    transCmplt[0] = 0
                return								#
#################################################################################

#Check data arrays to see if they contain data. If they do, store them into database
#################################################################################
    def dataToDB() :								#
        db = db_functions.get_connection()
        cursor = db.cursor()
        									#
        if len(noiseData) != 0 :						#
            for w in range(0,len(noiseData)) :					#
                db_functions.write_to_db_cursor(dbTable, noiseData[w], cursor)  #
        if len(pktData) != 0 :							#
            for w in range(0,len(pktData)) :					#
                db_functions.write_to_db_cursor(dbTable, pktData[w], cursor)	#
            linkfile.write(linkName + "\n")
        if len(ackData) != 0 :							#
            for w in range(0,len(ackData)) :					#
                db_functions.write_to_db_cursor(dbTable, ackData[w], cursor)	#

        cursor.close()
        db_functions.tear_down(db)
#################################################################################


    #############################################################################
    #				Dump Main					#
    #############################################################################
    dbmOffset = int8(-45)

    #Create arrays to store packet and noise data.
    noiseData = []
    pktData = []
    ackData = []
    linkName = str(sourceID) + "+" + str(thisMoteID)

    #Setup serial connection and active message
    retries = 3
    while retries > 0:
        try:
            if (len(port.split('/'))>1): #[1] == 'dev'):
                s = tos.Serial(port,speed,debug=False)     #(myPort, speed, debug=False)
                am = tos.SimpleAM(s)                                                    #Use the Active Message packets
            else:
                s = tos.SerialMIB600(port,debug=False)
                am = tos.SimpleAM(s)
        except Exception:
            pass
        else:
            break
        retries -= 1
        print "connection timedout, retrying (%s); retries %d" % (port, retries)
        #maybe connection timeouts are correlated temporally
        sleep(2)

    if retries == 0:
        debug = open("../log/%s/errorlog.txt" % dbTable, "a")
        print "CONNECTION TIMED OUT even with retries: %s" % port
        debug.write("Connection timed out for port %s\n" % port)
        debug.close()
    else:
    #Set channel and wait for motes to set channel
        setChannel()
        sleep(1.0/100.0)

    #If noise measurement requested, send noise command and listen for RSSI packets
        if (noise == 1):
            setNoise()

    #Set mote flags ready for transmission/reception
        moteRdyFlag[index] = 1				#Set flag for this mote to let the transmitter know receivers are ready to receive data
    #Begin transmit
        if (port == source_port):
            transmit()

    #Listen for packets
        listenSerialRxTx()

        dataToDB()

        moteRdyFlag[index] = 0

#################################################################################
#										#
#				Begin Program					#
#										#
#################################################################################
#References to port == node address in the nodes file

#Specify format for user
#print "Usage:", sys.argv[0], "57600 [<Experiment DB Table Name>] [<Number of motes>] [<network@255.255.255.255:10002>].......[<network@255.255.255.255:10002>] [<mote id for 1st IP address>].....[<mote id for nth IP address>] [<Source ID>] [<Destination ID>] [<IPI>] [<Number of packets>] [<Channel/Frequency>] [<Power>] [<Acks : 1 for yes no retransmit, acks-1 specifies number of retransmit attempts on failed ack (must be less than 254), 0 for no>] [<CSMA: 1 for CSMA on all packets, 2 for no CSMA, 3 for CSMA on first packet and then no CSMA>] [<Noise sampling: 1 for yes, 0 for no>] [<Noise sampling IPI (if you go really low, you will lose samples)>] [<Number of noise samples>]\n","For broadcasting, specify -1 as Destination ID\n"
print sys.argv
#Parse Values
speed = int(sys.argv[1])
dbTable = sys.argv[2]
nomotes = int(sys.argv[3])
sourceID = int(sys.argv[2*nomotes + 4])
destID = int(sys.argv[2*nomotes + 5])
ipi = int(sys.argv[2*nomotes + 6])
number = int(sys.argv[2*nomotes + 7])
channel = int(sys.argv[2*nomotes + 8])
power = int(sys.argv[2*nomotes + 9])
acks = int(sys.argv[2*nomotes + 10])
csma = int(sys.argv[2*nomotes + 11])
noise = int(sys.argv[2*nomotes + 12])

if noise == 1	:
    nipi = int(sys.argv[2*nomotes + 13])
    nnumber = int(sys.argv[2*nomotes + 14])
else:
    nipi = 0
    nnumber = 0

broadcastID = 0xFFFF
firstmoteID = 4 + nomotes
lastmoteID = 4 + (nomotes << 1)
#source_port, source_position, dest_port, dest_position = (0,0,0,0)

#Create mapping of ids to ports
#id_to_port = {}
#for i in range(nomotes):
#    id_to_port[int(sys.argv[i + 4 + nomotes])] = sys.argv[4 + i];
id_array = sys.argv[firstmoteID:lastmoteID]
for i in range(nomotes):
    id = int(id_array[i])
    if id == sourceID:
#Find sourceID port
        source_port = sys.argv[4 + i]
        source_position = i
    elif id == destID:
#Find destID port and position
        dest_port = sys.argv[4 + i]
        dest_position = i

#counts how many packets were received
counters = zeros(nomotes, 'i')
#status variables
moteRdyFlag = zeros(nomotes,'i')
transCmplt = ones(1,'i')					#Set to 0 once transmitter is done transmitting

#Open File in Folder linkdata to keep track of files containing received packet data from receiver motes
linkfile = open("../log/" + dbTable + "/links.txt", "a")

#start thread that records progress
thread.start_new_thread(record_progress, ())
threads = []
#Setup connection threads in the case of either broadcast or unicast
if destID == -1	:						
    destID = broadcastID					#Set destID for broadcast if specified 
    for i in range(nomotes):			                #Start connections to all motes in seperate threads if broadcast
        t = Thread(target=dump, args=(sys.argv[4 + i], int(id_array[i]), i))
        threads.append(t)
        t.start()
else	:							#Else find dest port for unicast
    #Start connections to only transmitter and receiver in separate threads for unicast
    t = Thread(target=dump, args=(dest_port, destID, dest_position))
    threads.append(t)
    t.start()
    t = Thread(target=dump, args=(source_port, sourceID, source_position))
    threads.append(t)
    t.start()

for t in threads:
    t.join()
terminate()
