# Copyright (c) 2008 Stanford University.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# - Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the
#   distribution.
# - Neither the name of the Stanford University nor the names of
#   its contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL STANFORD
# UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# @author Kannan Srinivasan <srikank@stanford.edu>
# @author Maria Kazandjieva <mariakaz@cs.stanford.edu>
# @author Edward Kim <edskim@stanford.edu>
# @author Mayank Jain <mayjain@stanford.edu>
# Created: 11/05/2008
# Modified: 04/11/2009

# script to plot the bimodal metric distribution graph

from numpy import *
from pylab import *
import os
import sys
import kwinter

gridWidth = 0.1
textSize = 28
curveWidth = 4
scatterbubblesize = 100
xaxispadding = 15
figwidth = 10
figheight = 7
gridcolor = '0.5'
gridtransparency = 0.5
font = 'sans-serif'

col = 'blue'
compute_flag = 0

def plotbimodality(dbTable, PACKETS, IPI):
	global compute_flag
	clf()

	# calls the function in kwall.py to generate the kwdist files for every IPI
	f = kwinter.calculateall(dbTable,PACKETS,IPI)
	kwdist = []
	indy = []
	bi = []

	numbers = []
	links = []

	#numbers.append(100)
	if (len(f)!=0):
		for i in range(0,len(f)):
		    kwdist.append(float(f[i].split()[0])) #add kw distance value to array
		    indy.append(float(f[i].split()[1])) #add distance from independent value to array
		    links.append(f[i].split()[2])
		for i in range(0,len(indy)):
			bimod = float(indy[i]-kwdist[i])/float(indy[i])
			bi.append(bimod) # calculate bimodality value
	
		bi.sort()
		
		arr = array(bi)

		if (compute_flag == 0):
			compute_flag = 1
			tablefile = open("../log/" + dbTable+"/tablefile", 'a')
			burstyperc = float(len(find(arr>=0.8)))*100.0/float(len(arr))
			temp1 = array(arr[find(arr>=-0.2)])
			indeperc = float(len(find(temp1<=0.2)))*100.0/float(len(arr))
			tablefile.write("beta: " + str(round(burstyperc,2)) + " " + str(round(indeperc,2)) + "\n")
			tablefile.close()

	
		value = []
		occurances = []
		percentage = []
		
		value.append(arr[0])
		occurances.append(1)
		j = 0
	
		for i in range(1,len(arr)):
			if (arr[i] > value[j]):
				j += 1
				value.append(arr[i])
				occurances.append(1)
			else:
				occurances[j] +=1
		total = 0
		for k in range(0, len(value)):
			total += occurances[k]
			percentage.append(100 - 100*(float(total)/float(len(arr))))

	fig = figure(1)
	fig.set_size_inches(figwidth, figheight)
	ax = fig.add_subplot(111)
	ax.xaxis.set_ticks_position('bottom')
	ax.yaxis.set_ticks_position('left')
	
	for tick in ax.xaxis.get_major_ticks():
		tick.set_pad(xaxispadding)
	for tick in ax.yaxis.get_major_ticks():
		tick.set_pad(xaxispadding)
	             
	
	subplots_adjust(left=0.2, right=0.9, bottom=0.23, top=0.96)
	grid(True,linewidth=gridWidth,color=gridcolor,alpha=gridtransparency)
	
	xticklabels = get(gca(), 'xticklabels')
	setp(xticklabels, weight='bold', fontsize=textSize, fontname=font)

	yticklabels = get(gca(), 'yticklabels')
	setp(yticklabels, weight='bold', fontsize=textSize, fontname=font)

	#print value, percentage
	if (len(f)!=0):
		value_plot = concatenate(([-1],value,[1]),0)
		percentage_plot = concatenate(([100.0], percentage,[0.0]),0)
		for i in range(0, len(value_plot)-1):
			plot([value_plot[i], value_plot[i+1]], [percentage_plot[i], percentage_plot[i]], '-', color = col, label='IPI='+str(IPI)+'ms', linewidth=4)
			plot([value_plot[i+1], value_plot[i+1]], [percentage_plot[i], percentage_plot[i+1]], '-', color = col, label='IPI='+str(IPI)+'ms', linewidth=4)
	else:
		text(0,50,"No intermediate links.", ha='center', va='center',
fontsize = 30, weight = 'bold')
		tablefile = open("../log/" + dbTable+"/tablefile", 'a')
		tablefile.write("beta: n/a n/a\n")
		tablefile.close()

	ylabel("Percentage of links", family=font, weight='bold', fontsize=32)

	xlabel(r"$\beta$", family=font, weight='bold', fontsize=42)
	
	axis([-1.01, 1.01, -1, 101] )
	
	#savefig("../reports/"+dbTable+"/plots/beta-ccdf.pdf", dpi=50)
	savefig("../reports/"+dbTable+"/plots/beta-ccdf.png", dpi=50)

#plotbimodality(PACKETS, IPIs, color)
