#!/usr/bin/env python

# Copyright (c) 2008 Stanford University.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# - Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the
#   distribution.
# - Neither the name of the Stanford University nor the names of
#   its contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL STANFORD
# UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# @author Kannan Srinivasan <srikank@stanford.edu>
# @author Mayank Jain <mayjain@stanford.edu>
# Created: 11/05/2008

import sys, time
import thread
import string
import threading
import numpy as np
#import db_functions
import MySQLdb
import random as blah
from math import *
from pylab import *
from matplotlib.pyplot import figure, show, rc, grid

def valid_points_exist():
    for tx in range(0,numMotes) :
	for rx in range(tx+1,numMotes) :
            txrxPair = str(moteIDs[tx]) + " " +str(moteIDs[rx])
            if txrxPair in linkETX :
                etxFrom = linkETX[txrxPair]
                etxTo = linkETX[revLink(txrxPair)]
                if abs(etxFrom - etxTo) > threshold :
                    return True
    return False

def revLink(linkForward) :
    r = linkForward.split()
    return str(r[1]) + " " + str(r[0])

def plotLine(x1,y1,etxTo,x2,y2,etxFrom) :

    xSteps = (float(x1)-float(x2))/(steps-1)
    ySteps = (float(y1)-float(y2))/(steps-1)
    colorSteps = (float(etxTo)-float(etxFrom))/(steps-1)
    xPoints = []
    yPoints = []
    colorPoints = []
    varRange = range(0,steps)

    for h in range(0,len(varRange)-1) :
        xPoints.append(x2 + (varRange[h]*xSteps))
        yPoints.append(y2 + (varRange[h]*ySteps))
        colorPoints.append(etxFrom + (varRange[h]*colorSteps))

    d = scatter(xPoints,yPoints,marker = 'o', c = colorPoints, cmap = hot(), vmin = 0, vmax = 10, s = 2, edgecolors = 'None')


def plotLink(txrxPair):
    a = txrxPair.split()
    moteTx = int(a[0])
    moteRx = int(a[1])


    theta1index = moteIDs.index(moteTx)
    theta2index = moteIDs.index(moteRx)

    angle1 = theta[theta1index]
    angle1Rad = radians(angle1)
    x1 = radius * cos(angle1Rad)
    y1 = radius * sin(angle1Rad)


    angle2 = theta[theta2index]
    angle2Rad = radians(angle2)
    x2 = radius * cos(angle2Rad)
    y2 = radius * sin(angle2Rad)

    etxTo = linkETX[txrxPair]
    etxFrom = linkETX[revLink(txrxPair)]

    #Filter out to show only ETX asymmetries defined by
    #ETX difference of 0.1 min
    #Atleast one ETX below 3
    if abs(etxFrom - etxTo) <= 0.1 :
        return
    if etxFrom > 3 and etxTo > 3:
        return

    if abs(etxFrom - etxTo) < threshold :
        return
    anyAsymm.append(1)
    plotLine(x1,y1,etxTo,x2,y2,etxFrom)


def plotLinksAll() :
    for tx in range(0,numMotes) :
        for rx in range(tx+1,numMotes) :
            if str(moteIDs[tx]) + " " +str(moteIDs[rx]) in linkETX :
                plotLink(str(moteIDs[tx]) + " " +str(moteIDs[rx]))
                

#################################################################################
#										#
#				Begin Program					#
#										#
#################################################################################

threshold = 0

links = []
moteIDs = []
etx = []
linkETX = {}
theta = []
positions = []
radius = 30

numMotes = 0
steps = 500
anyAsymm = []

def plot_etxasymm(dbTableName, t, prr, arr):
    global threshold, numMotes
    clf()
    threshold = t
    linklistfile = open('../log/' + dbTableName + '/links.txt', 'r')

#print len(prr), " ", len(arr)

#Create dictionary entries for each link and the respective prr
    etxIndex = 0
    for line in linklistfile:
        x = line.split('\n')
        y = x[0].split('+')
        
        tx = int(y[0])
        rx = int(y[1])
        linkForward = str(tx) + " " + str(rx)
        linkBack = str(rx) + " " + str(tx)
        
        if tx not in moteIDs : 
            moteIDs.append(tx)
        if rx not in moteIDs : 
            moteIDs.append(rx)

        if (prr[etxIndex]*arr[etxIndex]) > 0.1 :
            etx.append(float(1/(prr[etxIndex]*arr[etxIndex])))
        else :
            etx.append(10)

    #Create dictionary entry for the link found in the link list
        linkETX[linkForward] = etx[etxIndex]

    #Create dictionary entry for other direction of link if it does not exist
        if linkBack not in linkETX:
            linkETX[linkBack] = 10

        links.append(linkForward)
        etxIndex += 1

    linklistfile.close()

    numMotes = len(moteIDs)

    fig=figure(1)
    fig.set_facecolor('w')

## Checking to see if valid points exist
    if valid_points_exist():
	#Find the list of angles that node labels will be placed at.
	#Index of angles will match up with index of the mote ids
	for i in range(0,numMotes) :
	    theta.append(i * 360/numMotes)
	    xMote = radius * float(cos(radians(theta[i])))
	    yMote = radius * float(sin(radians(theta[i])))
	    xText = 1.1 * radius * float(cos(radians(theta[i])))
	
	    yText = 1.1 * radius * float(sin(radians(theta[i])))
	
	    moteName = moteIDs[i]
	    c = scatter( [xMote], [yMote], marker = 'o')
	    
	    text(xText,yText,str(moteName), ha = 'center', va = 'center', weight = 'bold')
	
	#Plot the links between motes
	plotLinksAll()
	axis([-1.2*radius, 1.2*radius, -1.2*radius, 1.2*radius])
	axis('off')
	
	#print linkETX
	#print "PRR: ", prr
	#print "ARR: ", arr
	#print links
	if len(anyAsymm) > 0:
	    colorbar().set_label("ETX")
	
	#savefig("../reports/" + dbTableName + "/plots/etxasymm.pdf", dpi=50)
	savefig("../reports/" + dbTableName + "/plots/etxasymm.png", dpi=50)
    else:
	axis([-1.2*radius, 1.2*radius, -1.2*radius, 1.2*radius])
	axis('off')
	text(0, 0, "No valid data points", ha = 'center', va = 'center', weight = 'bold', fontsize = 30)
	savefig("../reports/" + dbTableName + "/plots/etxasymm.png", dpi=50)

