#!/usr/bin/env python

# Copyright (c) 2008 Stanford University.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# - Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the
#   distribution.
# - Neither the name of the Stanford University nor the names of
#   its contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL STANFORD
# UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# @author Edward Kim <edskim@stanford.edu>
# @author Kannan Srinivasan <srikank@stanford.edu>
# @author Mayank Jain <mayjain@stanford.edu>
# Created: 11/05/2008

import sys, time
import thread
import string
import threading
import numpy as np
import prrcalc
#import db_functions
import MySQLdb
import random as blah
from math import *
from pylab import *
from matplotlib.pyplot import figure, show, rc, grid, cm
import matplotlib.colors as colors

def valid_points_exist():
    for tx in range(0,numMotes) :
	for rx in range(tx+1,numMotes) :
            txrxPair = str(moteIDs[tx]) + " " +str(moteIDs[rx])
            if txrxPair in linkPrr :
                prrFrom = linkPrr[txrxPair]
                prrTo = linkPrr[revLink(txrxPair)]
                if abs(prrFrom - prrTo) > threshold :
                    return True
    return False

def revLink(linkForward) :
    r = linkForward.split()
    return str(r[1]) + " " + str(r[0])

def plotLine(x1,y1,prrTo,x2,y2,prrFrom) :

    xSteps = (float(x1)-float(x2))/(steps-1)
    ySteps = (float(y1)-float(y2))/(steps-1)
    colorSteps = (float(prrTo)-float(prrFrom))/(steps-1)
    xPoints = []
    yPoints = []
    colorPoints = []
    varRange = range(0,steps)

    for h in range(0,len(varRange)-1) :
        xPoints.append(x2 + (varRange[h]*xSteps))
        yPoints.append(y2 + (varRange[h]*ySteps))
        colorPoints.append(prrFrom + (varRange[h]*colorSteps))

    d = scatter(xPoints,yPoints,marker = 'o', c = colorPoints, cmap = rygmap, vmin = 0, vmax = 1, s = 2, edgecolors = 'None')


def plotLink(txrxPair):
    a = txrxPair.split()
    moteTx = int(a[0])
    moteRx = int(a[1])


    theta1index = moteIDs.index(moteTx)
    theta2index = moteIDs.index(moteRx)

    angle1 = theta[theta1index]
    angle1Rad = radians(angle1)
    x1 = radius * cos(angle1Rad)
    y1 = radius * sin(angle1Rad)


    angle2 = theta[theta2index]
    angle2Rad = radians(angle2)
    x2 = radius * cos(angle2Rad)
    y2 = radius * sin(angle2Rad)

    prrFrom = linkPrr[txrxPair]
    prrTo = linkPrr[revLink(txrxPair)]

    #Set threshold for plotting
    if abs(prrFrom - prrTo) < threshold :
        return
    plotLine(x1,y1,prrTo,x2,y2,prrFrom)


def plotLinksAll() :
    for tx in range(0,numMotes) :
        for rx in range(tx+1,numMotes) :
            if str(moteIDs[tx]) + " " +str(moteIDs[rx]) in linkPrr :
                plotLink(str(moteIDs[tx]) + " " +str(moteIDs[rx]))
                

#################################################################################
#										#
#				Begin Program					#
#										#
#################################################################################


####################################################################################################################


#n = db_functions.create_table("llama")

#linklistfile = open('../log/llama/links.txt','w')

#for tx in range(2,10):
#  for rx in range(2,10):
#      if (tx!=rx):
#          num = blah.randint(1,100)
#          for i in range(1,num+1):
#              db_functions.write_to_db(n, str(tx) + " " + str(rx) + " " + str(i) + " -40.5 50 0 0")
#          linklistfile.writelines(str(tx) + "+" + str(rx) + "\n")

#linklistfile.close()
####################################################################################################################

threshold = 0
rygmap = colors.LinearSegmentedColormap( "rygmap", cm.datad['RdYlGn'], 256)

links = []
moteIDs = []
linkPrr = {}

theta = []
radius = 30

numMotes = 0
steps = 500

def plot_linkasymm(dbTableName, t, startRef, endRef, prr):
    global threshold, numMotes, theta, moteIDs, linkPrr, links
    clf()

    threshold = t
    linklistfile = open('../log/' + dbTableName + '/links.txt', 'r')

#Create dictionary entries for each link and the respective prr
    prrIndex = 0
    for line in linklistfile:
        x = line.split('\n')
        y = x[0].split('+')

        tx = int(y[0])
        rx = int(y[1])
        linkForward = str(tx) + " " + str(rx)
        linkBack = str(rx) + " " + str(tx)

        if tx not in moteIDs : 
            moteIDs.append(tx)
        if rx not in moteIDs : 
            moteIDs.append(rx)

    #Create dictionary entry for the link found in the link list
        linkPrr[linkForward] = prr[prrIndex]

    #Create dictionary entry for other direction of link if it does not exist
        if linkBack not in linkPrr:
            linkPrr[linkBack] = 0

        links.append(linkForward)
        prrIndex += 1

    linklistfile.close()
    numMotes = len(moteIDs)

    fig=figure(1)
    fig.set_facecolor('w')
    fig.set_size_inches(8,6)
## Checking to see if valid points exist
    if valid_points_exist():
	#Find the list of angles that node labels will be placed at.
	#Index of angles will match up with index of the mote ids
	for i in range(0,numMotes) :
	    theta.append(i * 360/numMotes)
	    xMote = radius * float(cos(radians(theta[i])))
	    yMote = radius * float(sin(radians(theta[i])))
	    xText = 1.1 * radius * float(cos(radians(theta[i])))
	
	    yText = 1.1 * radius * float(sin(radians(theta[i])))
	
	    moteName = moteIDs[i]
	    c = scatter( [xMote], [yMote], marker = 'o')
	    
	    text(xText,yText,str(moteName), ha = 'center', va = 'center', weight = 'bold')
	
	#Plot the links between motes
	plotLinksAll()
	axis([-1.2*radius, 1.2*radius, -1.2*radius, 1.2*radius])
	axis('off')
	#print linkPrr
	colorbar().set_label("PRR")
	#savefig("../reports/" + dbTableName + "/plots/linkasymm-" + startRef + "-" + endRef + ".pdf", dpi=50)
	savefig("../reports/%s/plots/linkasymm-%s-%s.png" % (dbTableName, startRef, endRef), dpi=50)
    else:
	axis([-1.2*radius, 1.2*radius, -1.2*radius, 1.2*radius])
	axis('off')
	text(0, 0, "No valid data points", ha = 'center', va = 'center', weight = 'bold', fontsize = 30)
	savefig("../reports/%s/plots/linkasymm-%s-%s.png" % (dbTableName, startRef, endRef), dpi=50)
