# Copyright (c) 2008 Stanford University.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# - Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the
#   distribution.
# - Neither the name of the Stanford University nor the names of
#   its contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL STANFORD
# UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# @author Kannan Srinivasan <srikank@stanford.edu>
# Created: 11/05/2008

from numpy import *
from pylab import *
import sys
import noisecalc
import os


gridWidth = 0.1
textSize = 28
curveWidth = 4
scatterbubblesize = 100
xaxispadding = 15
figwidth = 10
figheight = 7
gridcolor = '0.5'
gridtransparency = 0.5
font = 'sans-serif'

#color = ['#CC3366', '#CC3399',  '#990099',  '#0033FF', '#00CCFF']

def plotnoisedistr(DBTABLENAME):
	clf()

	noise_floor = noisecalc.calculateall(DBTABLENAME)
	
	frequency = {}
	unique_noise = []
	for x in noise_floor:
		if (x in frequency):
			frequency[x] += 1.00
		else:
			unique_noise.append(float(x)-0.5)
			frequency[x] = 1.00
	#print frequency.values()/(array(frequency.values()).sum())
	
	fig = figure(1)
	fig.set_size_inches(figwidth, figheight)
	ax = fig.add_subplot(111)
	ax.xaxis.set_ticks_position('bottom')
	ax.yaxis.set_ticks_position('left')
	
	for tick in ax.xaxis.get_major_ticks():
		tick.set_pad(xaxispadding)
	for tick in ax.yaxis.get_major_ticks():
		tick.set_pad(xaxispadding)
	             
	
	subplots_adjust(left=0.14, right=0.9, bottom=0.23, top=0.96)
	grid(True,linewidth=gridWidth,color=gridcolor,alpha=gridtransparency)
	
	xticklabels = get(gca(), 'xticklabels')
	setp(xticklabels, weight='bold', fontsize=textSize, fontname=font)

	yticklabels = get(gca(), 'yticklabels')
	setp(yticklabels, weight='bold', fontsize=textSize, fontname=font)
	
	#print unique_noise
	bar(unique_noise, frequency.values()/(array(frequency.values()).sum()), width=1.0, bottom=-0.01)	
	ylabel("Probability", family=font, weight='bold', fontsize=32)
	xlabel("Noise Floor", family=font, weight='bold', fontsize=32)
	
	axis([array(unique_noise).min()-0.1, array(unique_noise).max()+1.1, -0.01, 1.01] )

	#savefig("../reports/" + DBTABLENAME + "/plots/noise-distr.pdf", dpi=50)
	savefig("../reports/" + DBTABLENAME + "/plots/noise-distr.png", dpi=50)
	

#plotnoisedistr(color, DBTABLENAME)
