#! /usr/bin/env python
# Program that generates  the connectivity graph of the nodes at a certain point in the experiment.
# Has to be called within the swan-swat directory
# Once remote logging is implemented in 802.11, this file may become irrelevant (use 802.15.4 visualization instead)
# @author Jonathan Chu <jchu3@cs.stanford.edu>
import os, sys, time
import thread
import string
import threading
import numpy as np
import prrcalc
import MySQLdb
import random as blah
from pylab import cos, sin, text, scatter, gray, axis, savefig
from matplotlib.pyplot import figure, show, rc, grid
from math import radians
#usage: python progress.py transmitter_id node_file link_file image_file ipixnum
# @param transmitter_id the position of the node in the nodes file
#      Restriction: all nodes in the nodes file preceding the transmitter_id have transmitted
#      If no nodes are transmitting, put -1
# @param node_file the nodes file
# @param link_file the file containing the links
# @param image_file the file the image should be saved to
# @param ipixnum the result of ipi (in milliseconds) multipled by the number of packets

#writes to file to update the time to go.
#Note: progress based on which transmitter is transmitting. Assuming transmitters transmit in linear order.
#@param index the position in the nodes file of the transmitter
def update_time(index):
    current = os.getcwd()
    elapsed_file = open(current[0:current.find("swan-swat")] + "ui/report_progress/elapsed.txt", "w");
    elapsed_file.write(str((numNodes - index) * (2.0 * int(sys.argv[5]) / 1000 + 15)));
    elapsed_file.close();

node_file = open(sys.argv[2], "r")
link_file = open(sys.argv[3], "r")

nodes = []
for line in node_file:
    nodes.append(int(line.split()[0]))

theta = []
positions = []
radius = 30

numNodes = len(nodes)
steps = 200
circleSize = 20

#a node is transmitting
trans_id = int(sys.argv[1])
passed = False
if trans_id == -1:
    #no nodes are transmitting
    update_time(0)
    passed = True
else:
    update_time(trans_id)

#draw figure
fig=figure(1)
fig.set_facecolor('w')
for i in range(0,numNodes) :
    #calculations copied from prrcalc.py
    theta.append(i * 360/numNodes)
    xMote = radius * float(cos(radians(theta[i])))
    yMote = radius * float(sin(radians(theta[i])))
    xText = 1.1 * radius * float(cos(radians(theta[i])))
    
    yText = 1.1 * radius * float(sin(radians(theta[i])))
    
    nodeID = nodes[i]
    if passed:
        #nodes that haven't transmitted
        text(xText,yText,str(nodeID), ha = 'center', va = 'center', weight = 'bold', color = 'r')
    elif i == trans_id:
        #the transmitting node
        text(xText,yText,str(nodeID), ha = 'center', va = 'center', weight = 'bold', color = 'y')
        passed = True
        
    else:
        #nodes that have already transmitted
        text(xText,yText,str(nodeID), ha = 'center', va = 'center', weight = 'bold', color = "#4bff14")
        
    scatter( [xMote], [yMote], marker = 'o', c = 'b')

link_pairs = {}
for line in link_file:
    x = line.split('\n')
    y = x[0].split('+')
    forward = y[0] + " " + y[1]
    reverse = y[1] + " " + y[0]
    if reverse in link_pairs:
        #the current link has 2-way connectivity
        link_pairs[reverse] = True
    else:
        #the current link so far has only 1-way connectivity
        link_pairs[forward] = False
link_file.close()

for pair in link_pairs.keys():
    #calculations mostly taken from prrcalc.py
    y = pair.split()
    tx = int(y[0])
    rx = int(y[1])
    theta1index = nodes.index(tx)
    theta2index = nodes.index(rx)

    angle1 = theta[theta1index]
    angle1Rad = radians(angle1)
    x1 = radius * cos(angle1Rad)
    y1 = radius * sin(angle1Rad)
    
    angle2 = theta[theta2index]
    angle2Rad = radians(angle2)
    x2 = radius * cos(angle2Rad)
    y2 = radius * sin(angle2Rad)
    xPoints = []
    yPoints = []
    colorPoints = []
    xSteps = (float(x1)-float(x2))/(steps-1)
    ySteps = (float(y1)-float(y2))/(steps-1)
    # draw 2-way connectivity as a solid black line
    if link_pairs[pair] == True:
        for h in range(0, steps):
            xPoints.append(x2 + (h*xSteps))
            yPoints.append(y2 + (h*ySteps))
        scatter(xPoints,yPoints,marker = 'o', c = 'k', s = 1)
        #line = Line2D([x1, x2], [y1, y2], color = 'k')
        #line.draw(fig)
    else:
        #draw 1-way connectivity as a fading line
        colorSteps = 1.0 / (steps - 1)
        for h in range(0,steps) :
            xPoints.append(x2 + (h*xSteps))
            yPoints.append(y2 + (h*ySteps))
            colorPoints.append(1 - h*colorSteps)
        scatter(xPoints,yPoints,marker = 'o', c = colorPoints, cmap = gray(), vmin = 0, vmax = 1, s = 2, edgecolors = 'None')
axis([-1.2*radius, 1.2*radius, -1.2*radius, 1.2*radius])
axis('off')
#colorbar()
savefig(sys.argv[4], dpi=50)
