#!/usr/bin/env python
# Generates the connectivity graph
# jbenet@cs.stanford.edu

import numpy
import pylab
import matplotlib.pyplot as plt
import matplotlib.patches as ptch
import matplotlib.colors as colors
import matplotlib.colorbar as colorbar
import networkx as nx
import sys

rygmap = colors.LinearSegmentedColormap( "rygmap", plt.cm.datad['RdYlGn'], 256)

def gradient_plot(n1, n2, d1, d2):
    cs = []
    xs = []
    ys = []
    steps = 400
    dx = (float(n1[0])-float(n2[0]))/(steps)
    dy = (float(n1[1])-float(n2[1]))/(steps)
    dc = (float(d1) - float(d2))/(steps)
    
    for p in range(0, steps + 1):
        xs.append(n1[0] - p*dx)
        ys.append(n1[1] - p*dy)
        cs.append(d1    - p*dc)
        #plt.scatter([n1[0] - p*dx], [n1[1] - p*dy], marker='o', c=c, s=2, alpha= 1.0  + p*dc)
    # bug in matplotlib.... when cs = rgba array, array looses alpha...
    # print cs
    #plt.scatter(xs, ys, marker='o', c=cs, s=2, alpha=None)
    plt.scatter(xs, ys, c='g', marker='o', s=6)
    plt.scatter(xs, ys, c=cs, cmap=rygmap, edgecolors='None', vmin=0, vmax=1, marker='o', s=5)

def prrcolor(prr):
    if   d < 0.3: return 'r'
    elif d < 0.5: return 'y'
    elif d < 0.7: return 'b'
    elif d < 0.9: return 'c'
    else:         return 'g'


GRADIENT = True
FLOORS   = False

argv = []
for a in sys.argv:
    if a == "-c":   GRADIENT = False
    elif a == "-f": FLOORS = True
    else: argv.append(a)

if GRADIENT: fname = "gradient.graph.png"
else: fname = "graph.png"

if len(argv) > 1: 
    fname = argv[1]


fig = pylab.figure(1)
fig.set_dpi(40)
fig.set_figsize_inches(6, 4)
fig.subplots_adjust(left=0.0, right=1.0, bottom=0.0, top=1.0)

G = nx.MultiDiGraph()
G2 = nx.MultiDiGraph()

for line in sys.stdin.readlines():
    line = line.split(" ")
    if len(line) != 3: continue
    G.add_edge(line[0], line[1], float(line[2]))
    G2.add_edge(line[0], line[1], 1.0 - float(line[2]) )

elist  =[(u,v) for (u,v,d) in G.edges(data=True) ]
ecolors = []
for (u,v,d) in G.edges(data=True):
    ecolors.append( prrcolor(d) )


pos=nx.spring_layout(G2) # positions for all nodes

pos=nx.graphviz_layout(G)

if FLOORS:

    floors = {
    1:('r', [21,12,10,15,19,23,1,25]),
    2:('b', [26,31,22,30,18,2,11,16,9,7,32,0]),
    3:('y', [4,27,13,29,33,34]),
    4:('g', [24,6,5,14,17,20,28,3,8])}

    flnodes = {}
    patches = []
    for fl in floors:
        flnodes[fl] = filter(lambda x: int(x) in floors[fl][1], G.nodes())
        patches.append(ptch.Patch('k', facecolor=floors[fl][0]))

    # nodes
    nx.draw_networkx_nodes(G,pos,node_size=200, nodelist=flnodes[1] ,node_color=floors[1][0])
    nx.draw_networkx_nodes(G,pos,node_size=200, nodelist=flnodes[2] ,node_color=floors[2][0])
    nx.draw_networkx_nodes(G,pos,node_size=200, nodelist=flnodes[3] ,node_color=floors[3][0])
    nx.draw_networkx_nodes(G,pos,node_size=200, nodelist=flnodes[4] ,node_color=floors[4][0])

else:     
    nx.draw_networkx_nodes(G,pos,node_size=200,node_color='k')   

# edges

if GRADIENT:
    edges = {}
    for (u,v,d) in G.edges(data=True):
        if u not in edges: edges[u] = {}
        if v not in edges[u]: edges[u][v] = d

    for u in G.nodes():
        for v in G.nodes():
            if v > u: continue
            try: d1 = edges[u][v]
            except KeyError, e: d1 = 0.0
            try: d2 = edges[v][u]
            except KeyError, e: d2 = 0.0
            if d1 > 0 or d2 > 0:
                gradient_plot(pos[u], pos[v], d1, d2 )

#    cb = colorbar.ColorbarBaserygmap, boundaries=['0.00', '0.25', '0.50', '0.75', '1.00']
    #cax, kw = make_axes(fig.gca(), **kw)
    #cb = plt.colorbar(cax=cax, **kw)
    cb = plt.colorbar(shrink=0.5, aspect=30)
    cb.set_label('PRR')
#    cb.ax
    #c = pylab.colorbar()
    
else:
    nx.draw_networkx_edges(G,pos,edgelist=elist, edge_color=ecolors, width=3, alpha=0.5, style='dashed')
    

# labels
nx.draw_networkx_labels(G,pos,font_size=10,font_family='sans-serif', font_weight='bold', font_color='w')

if FLOORS:
    pylab.figlegend( patches, map(lambda x: "%s Floor" % x, ["First", "Second", "Third", "Fourth"]), 
                loc=1, labelsep=0.0, labelspacing=0.01, columnspacing=0.0)


plt.axis('off')
print plt.axis()

plt.savefig(fname) # save as png

#plt.show() # display


