# Copyright (c) 2008 Stanford University.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# - Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the
#   distribution.
# - Neither the name of the Stanford University nor the names of
#   its contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL STANFORD
# UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# @author Kannan Srinivasan <srikank@stanford.edu>
# @author Edward Kim <edskim@stanford.edu>
# Created: 11/05/2008

from numpy import *
from pylab import *
import math
import MySQLdb

sql = MySQLdb

def calculateall (dbTable, PACKETS, increment):
	prr = []
	avgrssi = []
	stdrssi = []
	avgcci = []
	stdcci = []
	#for asymmetric link calculations
	#array of arrays, first index corresponding to the time division, second index corresponding to the link prr for that division
	#ex. for an increment of 25, index 2 will count packets with sequence numbers between PACKETS*.25*2/100 and PACKETS*.25*3/100
	asymm_prr = []
	# highest sequence numbered packet to count towards the prr (indexed by the time division)
	asymm_end = []
	# array to store the counts of packets
	asymm_counters = []
	#whether or not to double count (when there is overlapping in the time divisions)
	asymm_equal = []
	# the range of sequence numbers to count in that time division
	asymm_range = []
	for i in range(0, 100, increment):
		next = PACKETS * (i+increment)/100.0
		asymm_prr.append([])
		asymm_end.append(math.floor(next))
		asymm_equal.append(next < PACKETS and math.ceil(next) == asymm_end[-1])
		asymm_range.append(asymm_end[-1] - math.ceil(PACKETS * i / 100.0) + 1)
		asymm_counters.append(0)
	asymm_len = len(asymm_prr)
	
	links=open('../log/' + dbTable + '/links.txt')						
	#prrfile = open('../metricdata/prrfile'+str(IPI)+'all.txt','w')
	db = sql.connect(host="localhost", user="root", passwd="", db="swat")
	cursor = db.cursor()
	for line1 in links:
            pkt = set()
	    #reset asymm counters
	    for i in range(asymm_len):
		    asymm_counters[i] = 0
	    link = line1.split('.')
	    tx = link[0].split('+')[0]
	    rx = link[0].split('+')[1]
	    cursor.execute("SELECT * FROM " + dbTable + " WHERE tx = " + str(tx) + " AND rx = " + str(rx) + " AND ack = 0 AND lqi < 255")
	    numrows = int(cursor.rowcount)
	    rssi = []
	    cci = []
	    for i in range(0, numrows):
	        x = cursor.fetchone()
		rssi.append(x[3])
		cci.append(x[4])
		sequence = int(x[2])
                if sequence not in pkt:
                    pkt.add(sequence)
		    #fill in values for link_asymm
		    for j in range(0, asymm_len):
			    if sequence <= asymm_end[j]:
				    asymm_counters[j] += 1
				    break
		    if sequence == asymm_end[j] and asymm_equal[j]:
			    asymm_counters[j + 1] += 1
			
            pktnum = len(pkt)

	    #print avgrssi

	    avgrssi.append(array(rssi).mean())
	    stdrssi.append(array(rssi).std())
	    avgcci.append(array(cci).mean())
	    stdcci.append(array(cci).std())
	    prr.append(float(pktnum)/float(PACKETS))
	    #add counters to asymm array
	    for i in range(asymm_len):
		    asymm_prr[i].append(asymm_counters[i] / asymm_range[i])
	    #prrfile.write(str(float(numrows)/float(PACKETS)) + " " + str(avgrssi) + " " + str(stdrssi) + " " + str(avgcci) + " " + str(stdcci) + " " + line1[:-1] + "\n") # packet reception ratio
	    #f.close()
	#prrfile.close()
	cursor.close()
	db.close()
	links.close()
	return prr, avgrssi, stdrssi, avgcci, stdcci, asymm_prr

def calcPRR (dbTable, PACKETS, startP, endP):
	prr = []
	
	links=open('../log/' + dbTable + '/links.txt')						
	db = sql.connect(host="localhost", user="root", passwd="", db="swat")
	cursor = db.cursor()

	for line1 in links:
            pkt = []
	    link = line1.split('.')
	    tx = link[0].split('+')[0]
	    rx = link[0].split('+')[1]

            startRow = math.ceil(PACKETS * startP / 100)
            endRow = math.floor(PACKETS * endP / 100)
	    cursor.execute("SELECT * FROM " + dbTable + " WHERE tx = " + str(tx) + " AND rx = " + str(rx) + " AND ack = 0 AND lqi < 255 AND seq >= " + str(startRow) + " AND seq <= " + str(endRow) )
	    numrows = int(cursor.rowcount)

            #Only count packets with unique seq numbers
	    for i in range(0, numrows):
	        x = cursor.fetchone()
                if str(x[2]) not in pkt:
                    pkt.append(str(x[2]))

            pktnum = len(pkt)
	    prr.append(float(pktnum)/(endRow - startRow))

	cursor.close()
	db.close()
	links.close()
	return prr

def calcARR (dbTable):
	arr = []
	
	links=open('../log/' + dbTable + '/links.txt')						
	db = sql.connect(host="localhost", user="root", passwd="", db="swat")
	#really only need one cursor...
	packcursor = db.cursor()
	ackcursor = db.cursor()

	for line1 in links:
            pkt = []
	    link = line1.split('.')

	    tx = link[0].split('+')[0]
	    rx = link[0].split('+')[1]


	    packcursor.execute("SELECT * FROM " + dbTable + " WHERE tx = " + str(tx) + " AND rx = " + str(rx) + " AND ack = 0 AND lqi < 255" )
	    packetsreceived = int(packcursor.rowcount)

            #Only count packets with unique seq numbers
	    for i in range(0, packetsreceived):
	        x = packcursor.fetchone()
                if str(x[2]) not in pkt:
                    pkt.append(str(x[2]))
            pktnum = len(pkt)

	    ackcursor.execute("SELECT * FROM " + dbTable + " WHERE tx = " + str(rx) + " AND rx = " + str(tx) + " AND ack = 1 AND lqi < 255" )
	    acksreceived = int(ackcursor.rowcount)

            #check logic - ek
            if pktnum != 0:
	        arr.append(float(acksreceived)/float(pktnum))

	ackcursor.close()
	packcursor.close()
	db.close()
	links.close()
	return arr
