# Copyright (c) 2008 Stanford University.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# - Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the
#   distribution.
# - Neither the name of the Stanford University nor the names of
#   its contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL STANFORD
# UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# @author Kannan Srinivasan <srikank@stanford.edu>
# @author Mayank Jain <mayjain@stanford.edu>
# Created: 11/05/2008
# Modified: 04/11/2009

from numpy import *
from pylab import *
from time import time
import sys

gridWidth = 0.1
textSize = 28
curveWidth = 4
scatterbubblesize = 100
xaxispadding = 15
figwidth = 10
figheight = 7
gridcolor = '0.5'
gridtransparency = 0.5
font = 'sans-serif'

#color = ['#CC3366', '#CC3399',  '#990099',  '#0033FF', '#00CCFF']
col = 'blue'
compute_flag = 0

def plotprrdistr(DBTABLENAME, PACKETS, IPI, NUMNODES, prrarr, avgrssi, stdrssi, avgcci, stdcci):
	global compute_flag
	clf()
	#print avgrssi
	#prrfile = open('../metricdata/prrfile'+str(IPI)+'all.txt')
	prr = zeros([NUMNODES*(NUMNODES-1)])
	prr[0:len(prrarr)] = prrarr[0:len(prrarr)]
	
	#index = 0
	#for line1 in prrfile:
	#    x = line1.split()
	#    prr[index] = float(x[0]) # packet reception ratio
	#    index += 1
	#prrfile.close()
	prr.sort()
	#print prr
	#arr = sorted(prr)
	#print range(len(prr)-1,-1,-1)
	#print "reversed prr is ", prr[range(len(prr)-1,-1,-1)]
	#arr = prr[range(len(prr)-1,-1,-1)]
	arr = prr
	
	if (compute_flag == 0):
		compute_flag = 1
		tablefile = open("../log/" + DBTABLENAME+"/tablefile",'w')
		perfect = 0
		good = 0
		intermediate = 0
		bad = 0
		dead = 0
		len_arr = len(arr)
		for i in range(len_arr):
			if arr[i] == 1.00:
				perfect += 1
			elif arr[i] > .9:
				good += 1
			elif arr[i] >= .1:
				intermediate += 1
			elif arr[i] > 0.0:
				bad += 1
			else:
				dead += 1
		perfect = round(perfect * 100.0 / len_arr, 2)
		good = round(good * 100.0 / len_arr, 2)
		intermediate = round(intermediate * 100.0 / len_arr, 2)
		bad = round(bad * 100.0 / len_arr, 2)
		dead = round(dead * 100.0 / len_arr, 2)
		
		tablefile.write("prr: %.2f %.2f %.2f %.2f %.2f\n" % (perfect, good, intermediate, bad, dead))
		tablefile.close()


	#print arr
	#arr.sort() # sort results
	#arr = sort(arr)

	value = []
	occurances = []
	percentage = []
	
	value.append(arr[0])
	occurances.append(1)
	j = 0

	for i in range(1,len(arr)):
		if (arr[i] > value[j]):
			j += 1
			value.append(arr[i])
			occurances.append(1)
		else:
			occurances[j] +=1

	total = 0
	for k in range(0, len(value)):
		total += occurances[k]
		#print array(occurances).sum()
		percentage.append(100*(float(total)/array(occurances).sum()))
		#percentage.append(100*(float(total)/(NUMNODES*(NUMNODES-1))))

		#print ("value " + str(value[k])+ " occurances " + str(occurances[k]) + " percentage " + str(percentage[k]))
	#print array(occurances).sum()
	fig = figure(1)
	fig.set_size_inches(figwidth, figheight)
	ax = fig.add_subplot(111)
	ax.xaxis.set_ticks_position('bottom')
	ax.yaxis.set_ticks_position('left')
	
	for tick in ax.xaxis.get_major_ticks():
		tick.set_pad(xaxispadding)
	for tick in ax.yaxis.get_major_ticks():
		tick.set_pad(xaxispadding)
	             
	
	subplots_adjust(left=0.2, right=0.9, bottom=0.23, top=0.96)
	grid(True,linewidth=gridWidth,color=gridcolor,alpha=gridtransparency)
	
	xticklabels = get(gca(), 'xticklabels')
	setp(xticklabels, weight='bold', fontsize=textSize, fontname=font)

	yticklabels = get(gca(), 'yticklabels')
	setp(yticklabels, weight='bold', fontsize=textSize, fontname=font)

	#print value, percentage
	value_plot = concatenate(([0],value,[1]),0)
	percentage_plot = concatenate(([0.0], percentage,[100.0]),0)
	
	for i in range(0, len(value_plot)-1):
		plot([value_plot[i], value_plot[i+1]], [percentage_plot[i], percentage_plot[i]], '-', color = col, label='IPI='+str(IPI)+'ms', linewidth=4)
		plot([value_plot[i+1], value_plot[i+1]], [percentage_plot[i], percentage_plot[i+1]], '-', color = col, label='IPI='+str(IPI)+'ms', linewidth=4)
	#plot(value, percentage, '-', color = col, label='IPI='+str(IPI)+'ms', linewidth=4)
	#plot(value, percentage, 'bx', markersize = 20)
	ylabel("Percentage of links", family=font, weight='bold', fontsize=32)
	xlabel("Reception Ratio", family=font, weight='bold', fontsize=32)
	
	axis([-0.01, 1.01, -1, 101] )
	
	#leg = legend(loc='lower left')
	#texts = leg.get_texts()
	#for te in texts:
	   #te.set_family(font)
	   #te.set_fontsize(24)

	#savefig("../reports/" + DBTABLENAME + "/plots/prr-cdf.pdf", dpi=50)
	savefig("../reports/" + DBTABLENAME + "/plots/prr-cdf.png", dpi=50)


#plotprrdistr(DBTABLENAME, PACKETS, RAWIPI, IPIs, NUMNODES)
