# Copyright (c) 2008 Stanford University.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# - Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the
#   distribution.
# - Neither the name of the Stanford University nor the names of
#   its contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL STANFORD
# UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# @author Kannan Srinivasan <srikank@stanford.edu>
# Created: 11/05/2008

from numpy import *
from pylab import *
import sys

gridWidth = 0.1
textSize = 28
curveWidth = 4
scatterbubblesize = 100
xaxispadding = 15
figwidth = 10
figheight = 7
gridcolor = '0.5'
gridtransparency = 0.5
font = 'sans-serif'

#color = ['#CC3366', '#CC3399',  '#990099',  '#0033FF', '#00CCFF']

def plotprrvscci(DBTABLENAME,prr, avgrssi, stdrssi, avgcci, stdcci):
	clf()
	#prrfile = open('../metricdata/prrfile'+str(IPI)+'all.txt')
	#prr = []
	#avgrssi = []
	#stdrssi = []
	
	#for line1 in prrfile:
	#    x = line1.split()
	#    prr.append(float(x[0])) # packet reception ratio
	#    avgrssi.append(float(x[1]))
	#    stdrssi.append(float(x[2]))
	#prrfile.close()

	fig = figure(1)
	fig.set_size_inches(figwidth, figheight)
	ax = fig.add_subplot(111)
	ax.xaxis.set_ticks_position('bottom')
	ax.yaxis.set_ticks_position('left')
	
	for tick in ax.xaxis.get_major_ticks():
		tick.set_pad(xaxispadding)
	for tick in ax.yaxis.get_major_ticks():
		tick.set_pad(xaxispadding)
	             
	
	subplots_adjust(left=0.14, right=0.9, bottom=0.23, top=0.96)
	grid(True,linewidth=gridWidth,color=gridcolor,alpha=gridtransparency)
	
	xticklabels = get(gca(), 'xticklabels')
	setp(xticklabels, weight='bold', fontsize=textSize, fontname=font)

	yticklabels = get(gca(), 'yticklabels')
	setp(yticklabels, weight='bold', fontsize=textSize, fontname=font)

	plot(avgcci, prr, 'o', markersize=6)
	hold('on')
	plot(array(avgcci)+array(stdcci), prr, '|', linewidth=2, color="gray")
	plot(array(avgcci)-array(stdcci), prr, '|', linewidth=2, color="gray")
	for i in range(0,len(prr)):
		plot([avgcci[i]+stdcci[i], avgcci[i]-stdcci[i]], [prr[i], prr[i]], '-', linewidth=2, color="gray")

	ylabel("Reception Ratio", family=font, weight='bold', fontsize=32)
	xlabel("CCI", family=font, weight='bold', fontsize=32)
	
	axis([49.99, 110.01, -0.01, 1.01])
	
	#leg = legend(loc='lower left')
	#texts = leg.get_texts()
	#for te in texts:
	   #te.set_family(font)
	   #te.set_fontsize(24)

	#savefig("../reports/" + DBTABLENAME + "/plots/prr-cci.pdf", dpi=50)
	savefig("../reports/" + DBTABLENAME + "/plots/prr-cci.png", dpi=50)


#plotprrvscci(DBTABLENAME, PACKETS, RAWIPI, IPIs, color)
