# script to generate bash script for running swan experiments
#
# @author Maria Kazandjieva, <mariakaz@cs.stanford.edu>
# @author Jonathan Chu <jchu3@cs.stanford.edu>
# @date Dec 23, 2008

import os
import sys

# check that both filename names have been specified, and no more
if (len(sys.argv) != 5):
	print "--- You need exactly four arguments ---"
	print " * config filename"
	print " * nodelist filename"
	print " * database table name"
	print " * generated script filename"
	print "--- Goodbye! --- "
	exit(1);

config = open(sys.argv[1], 'r')
nodelist = open(sys.argv[2], 'r')

config_data = {}

# associate fieldname with its values from config
for line in config:
	parts = line.split(" ", 1)
	config_data[parts[0]] = parts[1].split("\n")[0]

trials = int(config_data["num_trials:"])
channel = config_data["channel:"]
power = config_data["power_level:"]
num_packets = int(config_data["num_packets:"])
# ipi in microseconds 
ipi = []
for single in config_data["ipi:"].split():
	ipi.append(int(single) * 1000)
# to get real rate, divide number by 2 (each rate is represented as a string for convenience)
rate = []
for single in config_data["rate:"].split():
	rate.append(single)
packet_size = int(config_data["packet_size:"])

#default ssh key. Relative path not used because the statement containing it is executed in different folders.
ssh_key = os.getcwd().rsplit("/", 1)[0] + "/config/id_dsa.swan"

#get through the nodelist
nodes = []
nodeName = []

#some commands, like the setid command for swan-client, do not accept the user as input.
#for example, root@swan-ap is not valid, but swan-ap is.
for line in nodelist:
	parts = line.split()
	nodes.append(parts[1])
	temp = parts[1].split("@")
	if len(temp) == 1:
		nodeName.append(parts[1])
	else:
		nodeName.append(temp[1])
nodelist.close()

# time to generate the script
script = open(sys.argv[4], 'w')

script.write("#!/bin/bash\n\n")

#Kannan
for i in range(0, len(nodes)):
	script.write("ssh -i %s %s 'cp /usr/bin/gen_config_roofnet.SWAT.swan.sh /usr/bin/gen_config_roofnet.sh'\n" % (ssh_key, nodes[i]))
for i in range(0, len(nodes)):
	script.write("ssh -i %s %s 'write_handler swat.fopen 0'\n" % (ssh_key, nodes[i]))

for i in range(0, len(nodes)):
	script.write("ssh -i %s %s 'rm /tmp/log/swanmsg'\n" % (ssh_key, nodes[i]))

for i in range(0, len(nodes)):
	script.write("ssh -i %s %s '/usr/bin/roofnet'\n" % (ssh_key, nodes[i]))

script.write("sleep 30\n")
script.write("numpkts=%d\n" % num_packets)

#for (int i = 0; i < trials; i++)
script.write("for trial in %s; do\n" % " ".join([str(i) for i in range(1, trials + 1)]))

#for (int j = 0; j < powers; j++)
script.write("\tfor power in %s; do\n" % power) 

#for (int l = 0; l < channels; l++)
script.write("\t\tfor channel in %s; do\n" % channel)

progress_image = "../ui/progress.png"
link_file = "../log/%s/links.txt" % sys.argv[3]

#for (int k = 0; k < rates; k++)
for k in range(0, len(rate)):
	progress_info = ipi[k] * num_packets / 1000.0
#set node ids
	script.write("\t\t\techo '0 1 setid %s\n" % nodeName[0])
	
	for i in range (1, len(nodes)):
		script.write("0 %d setid %s\n" % (i+1, nodeName[i]))
	
	script.write("0 all run iwconfig ath2 txpower '$power'\n")
	script.write("0 all run iwconfig ath2 channel '$channel > ")
	script.write("cmdfile_txpower_channel\n./swan-client -f cmdfile_txpower_channel\n\n")
	script.write("\t\tpython ../metrics/progress.py -1 %s %s %s %d\n" % (sys.argv[2], link_file, progress_image, progress_info))
	script.write("\t\techo '0 1 setid %s\n" % nodeName[0])	

	for i in range (1, len(nodes)):
		script.write("0 %d setid %s\n" % (i+1, nodeName[i]))
	
	#script.write("0 all run swanLogger.init\n")
	#script.write("0 all run /etc/init.d/syslog-ng stop\n")
	script.write("0 all run write_handler swat.numpkts '$numpkts'\n")
	script.write("0 all run write_handler swat.ipi %d\n" % ipi[k])
	script.write("0 all run write_handler swat.rate %s\n" % rate[k])
	script.write("0 all run write_handler swat.power '$power'\n")

#running the script up to here takes about 2 minutes
#start each node, work the timing
	time = 10
	for i in range (0, len(nodes)):
	# open log files
		script.write("%d all run write_handler swat.fopen 1\n" % time)
	# start transmitter
		script.write("%d %d run write_handler swat.start 1\n" % (time, i+1))
	# update progress image
		script.write("%d local run python ../../metrics/progress.py %d ../%s ../%s ../%s %d\n" % 
			     (time, i, sys.argv[2], link_file, progress_image, progress_info))
	#update time
		time +=int(2.0*num_packets*ipi[k]/1000000) + 10
	#close all log files
		script.write("%d all run write_handler swat.fopen 0\n" % time)
	#retrieve and remove log files
		for j in range(0, len(nodes)):
			if j != i:
				script.write("%d local run scp -i %s %s:/tmp/swat_log swat_log_%d%d\n" % (time, ssh_key, nodes[j], i, j))
				script.write('%d local run ssh -i %s %s "rm /tmp/swat_log"\n' % (time, ssh_key, nodes[j]))
	# parse log files and put data into database
		script.write("%d local run python ../parse_files.py %s ../%s %d\n" % (time, sys.argv[3], sys.argv[2], i))

	common = "SWAT.allchannel.r%s.ch$channel.p$power" % rate[k]
#Kannan
	line = "' > "

	line += "cmdfile_" + common + "\n"
	line += "\t\t\tmkdir " + common + ".t$trial\n"
	line += "\t\t\tcd " + common + ".t$trial\n"
	line += "\t\t\t./../swan-client -f ../cmdfile_" + common + "\n"
	line += "\t\t\tcd ..\n"

	script.write(line)
	script.write("\t\techo 'rate %s ch '$channel' pow '$power\n" % rate[k])
script.write("\t\tdone\n")
script.write("\tdone\n")
script.write("done\n\n")

#clean up
script.write("for power in %s; do\n" % power)

script.write("\tfor rate in %s; do\n" % " ".join(rate))

script.write("\t\tfor channel in %s; do\n" % channel)

prefix = "SWAT.allchannel.r$rate.ch$channel.p$power"
script.write("\t\t\trm cmdfile_" + prefix + "\n")

script.write("\t\t\tfor trial in %s; do\n" % " ".join([str(i) for i in range(1, trials + 1)]))
script.write("\t\t\t\trmdir " + prefix + ".t$trial\n")
script.write("\t\t\tdone\n")
script.write("\t\tdone\n")
script.write("\tdone\n")
script.write("done\n")
script.write("rm cmdfile_txpower_channel\n")
# generate image one last time (for calculating.html)
script.write("python ../metrics/progress.py %d %s %s %s 0\n" % (len(nodes), sys.argv[2], link_file, progress_image))
script.close()
print "--- Script successfully generated ---"
print "filename: %s\n" % sys.argv[4]
