//This file is responsible for most of the dynamicity on config.html
//@author Jonathan Chu <jchu3@cs.stanford.edu>
//enables ACK selection
function enable_ACKS(){
	addOption(document.swat.Ack,"yes", "yes");
	addOption(document.swat.Ack,"no", "no");
	document.swat.Ack.disabled = false;
}
//disables ACK selection
function disable_ACKS(){
    removeAllOptions(document.swat.Ack);
    document.swat.Ack.disabled = true;
    document.swat.retry.disabled=true;
}
//802.15.4 selected, disable/enable appropriate fields
function selected802_15_4(){
    //show all 802.15.4-related fields
    var fields = document.getElementsByName("802_15_4");
    for (i = 0; i < fields.length; i++)
	fields[i].style.display = "table-row";
    //hide all 802.11-related fields
    fields = document.getElementsByName("802_11");
    for (i = 0; i < fields.length; i++)
	fields[i].style.display = "none";

    //properly set whether or not we want ACKs to be selectable
    if(document.swat.Transmission.value == 'broadcast')
	disable_ACKS()
	
    //change the power range
    document.getElementById("power_change").innerHTML = "31";
    //change the ipi range
    document.getElementById("ipi_change").innerHTML = "10";
    //change the help link for the nodes
    document.getElementById("node_popup").onclick=function(event){
	window.open('nodes.html', 'mywindow','width=400,height=330,resizable=no');
    };
}
//802.11 selected
function selected802_11(){
    var fields = document.getElementsByName("802_15_4");
    for (i = 0; i < fields.length; i++)
	fields[i].style.display = "none";

    fields = document.getElementsByName("802_11");
    for (i = 0; i < fields.length; i++)
	fields[i].style.display = "table-row";
    if (document.swat.Ack.disabled)
	enable_ACKS();
    document.getElementById("power_change").innerHTML = "18";
    rate_change();
    document.getElementById("node_popup").onclick=function(event){
	window.open('nodes_11.html', 'mywindow','width=400,height=330,resizable=no');
    };
}
/* rate has changed. Then so does the minimum ipi. */
function rate_change(){
    if (document.swat.rate.value == 1)
	document.getElementById("ipi_change").innerHTML = "15";
    else if (document.swat.rate.value == 11)
	document.getElementById("ipi_change").innerHTML = "5";
    else
	document.getElementById("ipi_change").innerHTML = "10";
}
/* update the estimate of the experiment run-time.
@return the run-time in minutes. */
function update_estimate(){
    var num_packets = parseInt(document.swat.num_packets.value);
    var ipi = parseInt(document.swat.ipi.value);
    var num_nodes = parseInt(document.swat.num_nodes.value);
    if (isNaN(num_packets) || isNaN(ipi) || isNaN(num_nodes))
	return NaN;
    var totalTime;
    if (document.swat.Interface.value == "802.11")
    {
	/*802.11 has a constant and proportional setup time */
	totalTime = 30 + 6 * num_nodes;
	//add on the estimate to run experiment (taken from gen-script.py)
	totalTime += 10 + num_nodes * (2.0 * num_packets * ipi / 1000 + 15);
    } else
    {
	var nipi;
	var num_noise;
	//802.15.4 needs the noise data
	if (document.swat.noise.value == "yes")
	{
	    nipi = parseInt(document.swat.num_samples.value);
	    num_noise = parseInt(document.swat.noisesample_rate.value);
	    if (isNaN(nipi) || isNaN(num_noise))
		return NaN;
	    nipi /= 1000.0;
	} else
	{
	    nipi = 0;
	    num_noise = 0;
	}
	//802.15.4 has a setup time dependent on the number of nodes
	totalTime = 10.8 * num_nodes;
	var round_time = 4.01 + num_packets*ipi/1000.0+nipi*num_noise + 5;
	//experiment runs differently depending on the transmission type
	if (document.swat.Transmission.value == "broadcast")
	{
	    totalTime += num_nodes * round_time;
	} else
	{
	    totalTime += num_nodes * (num_nodes - 1) * round_time;
	}
    }
    //return seconds
    return totalTime;
}
/* writes the estimate in the Estimate: field on config.html */
function print_estimate(){
    var container = document.getElementsByName("estimate")[0];
    var seconds = update_estimate();
    if (isNaN(seconds))
	container.innerHTML = "----";
    else
    {
	var value;
        seconds = seconds.toFixed(0);
	hours = parseInt(seconds / 3600);
	minutes = parseInt(seconds % 3600 / 60);
	seconds = seconds % 60;
	if (hours == 0)
	{
	    value = minutes + ":";
	    if (seconds < 10)
		value += "0";
	    value += seconds + " minutes";
	}
	else
	{
	    value = hours + ":";
	    if (minutes < 10)
		value += "0";
	    value += minutes + ":";
	    if (seconds < 10)
		value += "0";
	    value += seconds + " hours";
	    
	}
	container.innerHTML = value;
    }
}

function fillCategory(){ 
// this function is used to fill the category list on load
//removeAllOptions(document.swat.Interface);
addOption(document.swat.Interface, "802.11", "802.11", "");
addOption(document.swat.Interface, "802.15.4", "802.15.4", "");
//because 802.15.4 will be the first selected node
selected802_15_4();

removeAllOptions(document.swat.Transmission);
addOption(document.swat.Transmission, "broadcast", "broadcast", "");
addOption(document.swat.Transmission, "unicast", "unicast", "");

enable_ACKS();

document.swat.retry.disabled=true;
addOption(document.swat.Channel,"11", "11");
addOption(document.swat.Channel,"12", "12");
addOption(document.swat.Channel,"13", "13");
addOption(document.swat.Channel,"14", "14");
addOption(document.swat.Channel,"15", "15");
addOption(document.swat.Channel,"16", "16");
addOption(document.swat.Channel,"17", "17");
addOption(document.swat.Channel,"18", "18");
addOption(document.swat.Channel,"19", "19");
addOption(document.swat.Channel,"20", "20");
addOption(document.swat.Channel,"21", "21");
addOption(document.swat.Channel,"22", "22");
addOption(document.swat.Channel,"23", "23");
addOption(document.swat.Channel,"24", "24");
addOption(document.swat.Channel,"25", "25");
addOption(document.swat.Channel,"26", "26");

addOption(document.swat.Mote,"micaz", "micaz");
addOption(document.swat.Mote,"telosb", "telosb");
//if user reloaded page, may want to update estimate
print_estimate();
}

function SelectTX(){

removeAllOptions(document.swat.Ack);
if(document.swat.Transmission.value == 'unicast')
    enable_ACKS();
if(document.swat.Transmission.value == 'broadcast')
    disable_ACKS();
print_estimate();
}

function SelectNoise(){
if(document.swat.noise.value == 'yes'){
	document.swat.num_samples.disabled=false;
	document.swat.noisesample_rate.disabled=false;
}
if(document.swat.noise.value == 'no'){
	document.swat.num_samples.disabled=true;
	document.swat.noisesample_rate.disabled=true;
}
print_estimate();
}

function SelectRetry(){
if(document.swat.Ack.value == 'no'){
	document.swat.retry.disabled=true;
}
if(document.swat.Ack.value == 'yes'){
	document.swat.retry.disabled=false;
}
		
}

function SelectSubCat(){
// ON selection of category this function will work

removeAllOptions(document.swat.Channel);
addOption(document.swat.Channel, "", "channel", "");

if(document.swat.Interface.value == '802.15.4'){
addOption(document.swat.Channel,"11", "11");
addOption(document.swat.Channel,"12", "12");
addOption(document.swat.Channel,"13", "13");
addOption(document.swat.Channel,"14", "14");
addOption(document.swat.Channel,"15", "15");
addOption(document.swat.Channel,"16", "16");
addOption(document.swat.Channel,"17", "17");
addOption(document.swat.Channel,"18", "18");
addOption(document.swat.Channel,"19", "19");
addOption(document.swat.Channel,"20", "20");
addOption(document.swat.Channel,"21", "21");
addOption(document.swat.Channel,"22", "22");
addOption(document.swat.Channel,"23", "23");
addOption(document.swat.Channel,"24", "24");
addOption(document.swat.Channel,"25", "25");
addOption(document.swat.Channel,"26", "26");


removeAllOptions(document.swat.Mote);
addOption(document.swat.Mote,"micaz", "micaz");
addOption(document.swat.Mote,"telosb", "telosb");

selected802_15_4();
}

if(document.swat.Interface.value == '802.11'){
addOption(document.swat.Channel,"1", "1");
addOption(document.swat.Channel,"2", "2");
addOption(document.swat.Channel,"3", "3");
addOption(document.swat.Channel,"4", "4");
addOption(document.swat.Channel,"5", "5");
addOption(document.swat.Channel,"6", "6");
addOption(document.swat.Channel,"7", "7");
addOption(document.swat.Channel,"8", "8");
addOption(document.swat.Channel,"9", "9");
addOption(document.swat.Channel,"10", "10");
addOption(document.swat.Channel,"11", "11", "");

selected802_11();
}
print_estimate();
}
////////////////// 

function removeAllOptions(selectbox)
{
	var i;
	for(i=selectbox.options.length-1;i>=0;i--)
	{
		//selectbox.options.remove(i);
		selectbox.remove(i);
	}
}


function addOption(selectbox, value, text )
{
	var optn = document.createElement("OPTION");
	optn.text = text;
	optn.value = value;
	optn.selected = true;
	selectbox.options.add(optn);
}