#!/usr/bin/env python

# Copyright (c) 2008 Stanford University.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# - Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the
#   distribution.
# - Neither the name of the Stanford University nor the names of
#   its contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL STANFORD
# UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# @author Maria Kazandjieva <mariakaz@cs.stanford.edu>
# @author Jonathan Chu <jchu3@cs.stanford.edu>
# @date: 11/12/2008


import get_databases
#import commands
#import string, time
import cgi
import os, sys
import write_report
import subprocess
import subprocess
from threading import Thread, Lock
from time import sleep
import upload
from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer

sys.path.append(os.getcwd().rsplit("/", 1)[0] + "/experimentfiles")
from db_functions import add_table


table_name = "" #keep the table name being worked on
cmd = "" #command to send to the thread
args = "" #arguments to that command
black_hole = open("/dev/null", "r+")

#is the handler currently running a test? This variable makes sure only one test is running at a time
performingTest = False
mutex = Lock()
mutex_11 = Lock() #802.11 lock on progress bar
# estimated run-time length
run_time = 0
#how many pixels to increment the progress bar by every 3 seconds
increment = 0

#variables below apply only to 802.15.4 (and are not supported ... yet)
live_nodes = ""
total_nodes = "" 
total_links = ""
done_links = ""
# HTTP server to handle SWAT UI
# split between serving GET and POST request
	
class MyHandler(BaseHTTPRequestHandler):
	
	def do_GET(self):
		global table_name, live_nodes, total_nodes, total_links, done_links
		global mutex, performingTest
		try:
			if self.path.endswith("programming.html"):
				prog_status = open("../log/" + table_name + "/status.txt", "r")
				status = prog_status.readline()
				s = status.split()
				run_progress = prog_status.readline()
				rs = run_progress.split()
				
				# keep track of whether all desired nodes were
				# programmed, or just part of them;
				# will be used later to provide more detailed
				# feedback to user
				if (s[1] == "success" or s[1] == "partial"):
					if (s[1] == "partial"):
						live_nodes = s[2]
					else:
						live_nodes = total_nodes
					
					# running has completed with 'success'
					# status so we are ready to auto
					# generate the report
					if (rs[1] == "success"):
						report(table_name, self)
					
					# still running, in which case just
					# reload the current page	
					else:
						write_programming(self)
						
				# programming has failed, so inform the user	
				elif (status.split()[1] == "fail"): 
					write_page(self, "prog_fail.html")
				
				# still programming, so just reload page	
				else:
					write_programming(self)

			# server various java script files
			elif self.path.endswith("js"):
				# identify script to server
				to_serve = self.path[1:] 
				write_page(self, to_serve, 'text/javascript')

			# load page while metric calculation is running	
			elif self.path.endswith("calculating.html"):
				# open file to check up on the status of the
				# metric calculation
				prog_status = open("../log/" + table_name + "/status.txt", "r")
				temp = prog_status.readline()
				temp = prog_status.readline()
				status = prog_status.readline()
				
				# if metrics have completed
				if (status.split()[1] == "success"):
					# create the report
					write_report.gen_report(table_name)
					# remove the progress picture (clean up data structures)
					remove_progress()
					# send it back to the user
					write_page(self, "../reports/" + table_name + "/report.html")

				# if metric calculation failed, alert the user	
				elif (status.split()[1] == "fail"):
					remove_progress()
					write_page(self, "metric_fail.html")
					
				# metric calculation has not completed so just
				# refresh the page		
				else:
					write_page(self, "calculating.html")
					
			# serve the data report page, the user will see an list
			# of all available databases			
			elif self.path.endswith("data.html"):
				# get all existing table names, for the user to choose from
				get_databases.get_dbs()
				# serve the page
				write_page(self, "data.html")

			# serve	the data sharing page, user will see a list of
			# tables she can share
			elif self.path.endswith("share.html"):
				# get all existing table names, for the user to choose from
				get_databases.get_dbs()
				# serve the page
				write_page(self, "share.html")
				
			# server the index		
			elif self.path.endswith("/"):
				write_page(self, "index.html")
			
			# load all the images that go in the final report	
			elif self.path.find("/plots")>=0:
				to_serve = self.path
				write_page(self, "../reports/" + table_name + to_serve)
				
			# load the printer-friendly version of the final
			# report, so people can print and pdf it	
			elif self.path.endswith("/printer.html"):
				to_serve = self.path[1:]
				write_page(self, "../reports/" + table_name + "/"  + to_serve)
			#create the page that changes the experiment run-time
			elif self.path.endswith("time.html"):
				self.wfile.write('''<html><head>
<meta http-equiv="refresh" content="3; url=time.html" />
<link rel="stylesheet" type="text/css" href="css-style.css" />
</head><body id="text">%s</body></html>'''
						 % time_elapsed())
			#load latex table text
			elif self.path.endswith("latex_table.tex"):
				write_page(self, "../log/" + table_name + "/latex_table.tex")
			# load other html and jpeg pages on demand
			else:
				to_serve = self.path[1:]
				parts = to_serve.split('.')
				
				if parts[1] == 'jpg':
					write_page(self, to_serve, "image/jpg")
				elif parts[1] == 'css':
					write_page(self, to_serve, "text/css")
				elif parts[1] == 'xml':
					write_page(self, to_serve, "text/xml")
				elif parts[1] == 'swf':
					write_page(self, to_serve, "application/x-shockwave-flash")
				else:
					write_page(self, to_serve)
		except IOError:
			self.send_error(404,'File Not Found: %s' % self.path)
			
	# POST method for sumitting configuration parameters and generating
	# reports for a specific database		
	def do_POST(self):
		global table_name, cmd, args, performingTest, run_time, increment
		name = ""
		# get the data from all fields
		try:
			ctype, pdict = cgi.parse_header(self.headers.getheader('content-type'))
			if (ctype == 'multipart/form-data'):
				query=cgi.parse_multipart(self.rfile, pdict)
				
			# if data came from the data report section, we know we
			# are about to generate report.
			if (self.path.endswith("data.html")):
				table = query.get("databases")[0]
				table_name = to_database(table)

				prog_status = open("../log/" + table_name + "/status.txt", "r")
				status = prog_status.readline()
				status = prog_status.readline()
				status = prog_status.readline()
				# if metrics have completed
				if (status.split()[1] == "success"):
					# create the report
					write_report.gen_report(table_name)
					# send it back to the user
					write_page(self, "../reports/" + table_name + "/report.html")
				# if metric calculation failed, regenerate data
				elif (status.split()[1] == "fail"):
					write_page(self, "metric_fail.html")
				# metric calculation has not completed
				else:
					if performingTest:
						#if another test is currently running, just take the user back to the home page.
						write_page(self, "index.html")
					else:
						performingTest = True
						report(table_name, self)
			
			# the POST request came from the config page, so we must
			# save all that in the config file; or if the user
			# uploaded the files just load them in and use them			
			elif (self.path.endswith("share.html")):
				# do something
				table = query.get("databases")[0]
				table_name = to_database(table)
				expname = table_name.partition("_")[0]
				retcode = upload.tar_and_upload(table_name, expname)
				if retcode == 0:
					f = open("uploaded.html")
					self.wfile.write(f.read())
					f.close()
				else:
					f = open("upload_failed.html")
					self.wfile.write(f.read())
					f.close()
				
			else:
				#if experiment is already runnning, don't run another experiment
				if performingTest:
					write_programming(self)
					return

				# using preexisting files
				if (query.get("submit1") != None):
					filedata = query.get("configfile")[0].strip()
					parts = filedata.split("\n")

					config_data = {}
					for line in parts:
						split = line.split()
						config_data[split[0]] = split[1]

					name = config_data["experiment_name:"]
					interface = config_data["interface:"]
					num_packets = config_data["num_packets:"]
					ipi = config_data["ipi:"]
					num_nodes = config_data["num_nodes:"]
					if config_data["noise_sample:"] == "yes":
						num_samples = config_data["num_samples:"]
						noisesample_rate = config_data["sample_ipi:"]
					tx = config_data["transmission:"]
					
					userfile = open ("../config/%s.config" % name, 'w')
					userfile.write(filedata)
					nodesfile = query.get("nodesfile1")[0].strip()
					nodes_name = "../config/nodes_%s.txt" % name
					nodes = open(nodes_name, 'w')
					if interface == "802.11":
						modify_nodes_file(nodes, nodesfile)
					else:
						nodes.write(nodesfile)
					keys = query.get("sshkey1")
					if (keys[0] != ""):
						keyfile = open("../config/id_dsa.swan", "w")
						keyfile.write(keys[0])
						keyfile.close()
					
				# new configuration
				else:
					filename = query.get('filename')
					userfile = open ("../config/" + filename[0] +".config", 'w')
					userfile.write("experiment_name: " + filename[0])
					
					nodes_name = "../config/nodes_" + filename[0] + ".txt"
					nodes = open(nodes_name, 'w')
					name = filename[0]
				
					testbed = query.get('testbed')
					userfile.write('\ntestbed: ' + testbed[0])	

					interface = query.get('Interface')[0]
					userfile.write("\ninterface: " + interface)
					
					channel = query.get('Channel')
					userfile.write("\nchannel: " + channel[0])
					
					powerlevel = query.get('Power')
					userfile.write("\npower_level: " + powerlevel[0])

					if (interface == "802.15.4"):
						mote = query.get('Mote')
						userfile.write("\nmote: " + mote[0])
						
					num_nodes = query.get('num_nodes')[0]
					userfile.write("\nnum_nodes: " + num_nodes)

					nodesfile = query.get("nodesfile2")[0].strip()

					keys = query.get("sshkey2")
					if interface == "802.11":
						modify_nodes_file(nodes, nodesfile)
						if keys[0] != "":
							keyfile = open("../config/id_dsa.swan", "w")
							keyfile.write(keys[0])
							keyfile.close()					
					else:
						nodes.write(nodesfile)
					num_packets = query.get('num_packets')[0]
					userfile.write("\nnum_packets: " + num_packets)
					ipi = query.get('ipi')[0]
					userfile.write("\nipi: " + ipi)

					if (interface == "802.11"):
						csma = "off"
						tx = "broadcast"
						noise = "no"
					else:
						csma = query.get('csma')[0]
						tx = query.get('Transmission')[0]
						noise = query.get('noise')[0]
					userfile.write("\ncsma: " + csma)
					userfile.write("\ntransmission: " + tx)
					if (tx == "unicast"):
						ack = query.get('Ack')
						userfile.write("\nack: " + ack[0])
						if (ack[0] == "yes"):
							retry = query.get("retry")
							userfile.write("\nmax_retries: " + retry[0])
					
					userfile.write("\nnoise_sample: " + noise)
					
					if (noise == "yes"):
						num_samples = query.get('num_samples')[0]
						userfile.write("\nnum_samples: " + num_samples)
						noisesample_rate = query.get("noisesample_rate")[0]
						userfile.write("\nsample_ipi: " + noisesample_rate)

					if (interface == "802.11"):
						rate = query.get('rate')
						userfile.write("\nrate: " + str(int(float(rate[0]) * 2)));

						packet_size = query.get('packet_size')
						userfile.write("\npacket_size: " + packet_size[0])

						userfile.write("\nnum_trials: 1")

				userfile.close()
				nodes.close()
				# create custom page
				create_custom_prog(interface)
				#tell the visualization it shouldn't switch to another page
				try:
					os.rename("report_progress/keepimg.jpg", "report_progress/keep_img.jpg")
				except OSError:
					pass
				#modify node list to include reverse hex ip address
				write_nodelist(num_nodes)
				#update time to run experiment. These formulas are the same as in params.js
				common = int(num_packets) * int(ipi) / 1000.0
				if interface == "802.11":
					run_time = 30.0 + 5.5 * int(num_nodes)
					increment = 150.0 / run_time * 3
					run_time += 10 + int(num_nodes) * (2.0 * common + 15)
				else:
					common += 9.01
					try:
						common += int(num_samples)*int(noisesample_rate)/1000.0
					except NameError:
						pass
					run_time = int(num_nodes) * common
					if tx == "unicast":
						run_time *= (int(num_nodes) - 1)
					run_time += 10.8 * int(num_nodes)
				# resets progress to 0
				write_progress("0", str(run_time))
				# remove picture (if needed)
				remove_progress()				
				#create database table (avoid race conditions)
				table_name = add_table(name, interface)
				# start the experiment by calling programNodes
				cmd = "./../experimentfiles/programNodes"
				args = name + " nodes_" + name + " " + table_name
				performingTest = True
				write_programming(self)
				mutex.release()
		except:
			pass

#writes out the ids of nodes used in the visualization
def write_nodelist(num_nodes):
	nodes = open("report_progress/temp_node.txt", "w")
	nodes.write("id\n")
	nodes.close()
	#write expected number of nodes to file
	numbers = open("report_progress/expected.txt", "w")
	numbers.write("header\n%s\n" % num_nodes)
	numbers.close()

#gear the programming.html page to 802.11 or 802.15.4.
#This function can be removed once remote logging is integrated into 802.11
def create_custom_prog(interface):
	target_file = open("programming.html", "w")
	original = open("mock_programming.html", "r")
	is_802_11 = interface == "802.11"
	for line_new in original:
		line = line_new.strip()
		result = ""
		#802.11 and 802.15.4 have different headers
		if line == "%%in_header%%":
			if is_802_11:
				result = '''<meta http-equiv="refresh" content="3; url=programming.html" />
<style type="text/css">
#inner {
%%width%%
background-color: #990000;
}
.row {
  position:relative;
  z-index:1;
  height: 1.25em;
  width:700px;
  border-top: 1px solid gray;
  border-left: 1px solid gray;
  border-right: 1px solid gray;
  padding-left:.25em;
}
.key {}
.value {
  position:absolute;
  left:200px;
}
.images {
  position:relative;
  top:.625em;
}
#outer {
  width:150px;
  border:2px solid #800000;
}
.progressBar {
  height:25px;
}
</style>\n'''
		#802.15.4's programming.html does not refresh, so we need another way to update the time elapsed
		elif line == "%%iframe%%":
			if not is_802_11:
				result = '<iframe src="report_progress/keep_img.jpg" onload="finished()" id="is_done" style="display:none"></iframe>\n'
		elif line == "%%javascript%%":
			if is_802_11:
				result = '''function image_load(){
  document.getElementById("space").innerHTML = '<br />Legend: <br />'+
'<div class="row"><span style="color:#4bff14" class="key">5</span>'+
'<span class="value">Node 5 has already transmitted</span></div>'+
'<div class="row"><span style="color:red" class="key">5</span>'+
'<span class="value">Node 5 has not transmitted</span></div>'+
'<div class="row"><span style="color:#dee03c" class="key">5</span>'+
'<span class="value">Node 5 is transmitting</span></div>'+
'<div class="row"><img src="black_line.png" alt="solid line" class="images" />'+
'<span class="value">Both nodes sent and received the other&#39;s packets.</span></div>'+
'<div class="row" style="border-bottom: 1px solid gray; height:3em"><img src="fading.png" alt="fading line" class="images" />'+
'<span class="value">Node at darker end has sent packets to node at lighter end, but not vice versa.</span></div>';
};
function image_error(){
   document.getElementById("space").innerHTML = '<div id="outer" class="progressBar"><div id="inner" class="progressBar"></div></div>';
};\n'''
			else:
				result = '''var done = document.getElementById("is_done");
function finished(){
   /* data gathering is done when the image is renamed, giving a "image not found" error */
   if (done.contentDocument.title == "Error response")
      window.location.replace("http://127.0.0.1:8080/programming.html");
}
function check_switch()
{
   done.src="report_progress/keep_img.jpg";
}
setInterval("check_switch();", 3000);\n'''
		#802.15.4 uses iframe with swf file, 802.11 uses html/css to visualize data
		elif line == "%%method%%":
			if is_802_11:
				result = '''<b><img src="progress.png" alt="Initializing nodes:" onerror="image_error()" onload="image_load()" /></b>
<div id="space"></div>\n'''
			else:
				result = '<iframe src="/report_progress/Animation.html" width="950" height="750" frameborder="0" scrolling="no"></iframe>\n'
		elif not is_802_11 and line == "%%time%%":
			result = '<iframe src="/time.html" frameborder="0" width="200" height="25" scrolling="no"></iframe>\n'
		else:
			result = line_new
		target_file.write(result)
	target_file.close()
	original.close()

# create a report 
def report(table_name, self):
	global cmd, args, mutex
	write_page(self, "calculating.html")
	cmd = "./../experimentfiles/plotMetrics.py"
	args = table_name
	mutex.release()

#convenience function that returns a page to the user
def write_page(self, page_url, content_type = 'text/html'):
	f = open(page_url, "rb")
	self.send_response(200)
	self.send_header('Content-type', content_type)
	self.end_headers()
	self.wfile.write(f.read())
	f.close()

#converts ip adddress from dotted notation to reverse hexidecimal
def dotted_convert(dotted):
	four = dotted.split(".")
	hex_str = ""
	for i in range(3, -1, -1):
		hex_str += "%.2x" % int(four[i])
	return hex_str

#because we don't want the user to enter the hex of the ip-address as well, we
#add it in behind the scenes
#@param nodes_file the file handle that the data will be written to
#@param data the user input data
def modify_nodes_file(nodes_file, data):
	lines = data.split("\n")
	parts = lines[0].split()
	nodes_file.write("%s   %s" % (lines[0], dotted_convert(parts[2])))

	for line in lines[1:]:
		parts = line.split()
		nodes_file.write("\n%s   %s" % (line, dotted_convert(parts[2])))

#convenience function that transforms the human readable table name into the database table name
def to_database(readable):
	parts = readable.split()
	time_parts = parts[5].split(":")
	time = time_parts[0] + "_" + time_parts[1] + "_" + time_parts[2]
	expname = parts[0][:-1]
	day = parts[2][:-1]
	table_name = expname + "_" + parts[1] + "_" + day + "_" + parts[3] + "_" + time
	if parts[6] == "(802.11)":
		return table_name + "_1"
	else:
		return table_name + "_0"

# function to run a subprocess specified by cmd, with arguments args
# this is used to run the programNodes and plotMetrics scripts
def spin_off():
	global cmd, args, performingTest, mutex, mutex_11, table_name, black_hole
	while True:
		mutex.acquire()
		#sleeping may allow a page to load
		sleep(1)
		plotMetrics = cmd.endswith("plotMetrics.py")
		#if 802.11 is chosen, run a specialized thread (by releasing a lock) that
		#updates progress
		if not plotMetrics and table_name[-1] == "1":
			mutex_11.release()
		subprocess.call([cmd, args])#, 0, None, None, black_hole, black_hole)
		#finished with plotMetrics. Now no experiments are running
		if plotMetrics:
			performingTest = False

#thread that moves 802.11's progress bar
def init_802_11_progress():
	global mutex_11, run_time, increment
	while True:
		mutex_11.acquire()
		progress = 0
		while progress < 150 - increment:
			sleep(3)
			progress += increment
			run_time -= 3
			write_progress(str(progress), str(run_time))
		sleep(3)
		run_time -= 3
		write_progress("150", str(run_time))
			

#write progress data to files
def write_progress(data, time_to_run):
	progress_file = open("report_progress/initialize_data", "w")
	progress_file.write(data)
	progress_file.close()
	elapsed_file = open("report_progress/elapsed.txt", "w")
	elapsed_file.write(time_to_run)
	elapsed_file.close()

#remove progress file
def remove_progress():
	try:
		os.remove("progress.png")
	except OSError:
		pass
#returns the time elapsed in a human readable format
def time_elapsed():
	elapsed_file = open("report_progress/elapsed.txt", "r")
	
	try:
		seconds = int(float(elapsed_file.readline()))
		hours = seconds / 3600
		minutes = seconds % 3600 / 60
		seconds = seconds % 60
		if hours == 0:
			number = "%d:%.2d minutes" % (minutes, seconds)
		else:
			number = "%d:%.2d:%.2d hours" % (hours, minutes, seconds)
	except ValueError:
		number = "N/A"
	elapsed_file.close()
	return number
#send out the programming page. It has some user-specific data, so
# we need to treat it specially
def write_programming(self):
	f = open("programming.html", "r")
	self.send_response(200)
	self.send_header('Content-type', "text/html")
	self.end_headers()
	line = f.readline()
	while line != "":
		#replace the custom line with data. This data comes from the file below, which is a single number
		# representing the width of the expanding inner div (simulating progression)
		if line == "%%width%%\n":
			progress_file = open("report_progress/initialize_data", "r")
			number = progress_file.readline()
			progress_file.close()
			self.wfile.write("width:%spx;\n" % (number))
			line = f.readline()
		elif line == "%%time%%\n":
			self.wfile.write(time_elapsed() + "\n")
			#optimization: since %%time%% is the last placeholder, just read the rest of the file and write it out
			self.wfile.write(f.read())
			line = ""
		else:
			self.wfile.write(line)
			line = f.readline()
	f.close()

# main functin to run webserver
# could change port 8080 if user wants to					
def main():
	global mutex, mutex_11
	try:
		port = 8080
		server = HTTPServer(('', port), MyHandler)
		print '\n*****************************************************'
		print 'SWAT server started ...'
		print 'Open a web browser and go to localhost:'+ str(port)
		print 'To close server, use <Ctrl+c>'
		print '*****************************************************\n'

		mutex_11.acquire()
		mutex.acquire()
		#thread that calls external programs
		t = Thread(target = spin_off, args = ())
		t.daemon = True
		t.start()
		#thread that runs 802.11's progress bar
		otherT = Thread(target = init_802_11_progress, args = ())
		otherT.daemon = True
		otherT.start()

		server.serve_forever()
	except KeyboardInterrupt:
		print '^C received, shutting down server'
		server.socket.close()
		exit(0)
		
if __name__ == '__main__':
	main()
