#!/usr/bin/env python
# Copyright (c) 2008 Stanford University.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# - Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the
#   distribution.
# - Neither the name of the Stanford University nor the names of
#   its contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL STANFORD
# UNIVERSITY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# @author Maria Kazandjieva <mariakaz@cs.stanford.edu>
# @author Jonathan Chu <jchu3@cs.stanford.edu>
# Created: 11/05/2008

def gen_report(db_name):
	ack = "n/a"
	numnoisesamples = "n/a"
	noiseipi = "n/a"
	motetype = "n/a"
	maxretries = "n/a"

	uncomment_flag = 0

	#print("writing to report")
	f = open("mock_report.html", "r")
	parts = db_name.split("_")
	expname = parts[0]
	date = "%s %s, %s" % (parts[1], parts[2], parts[3])
	time = "%s:%s:%s" % (parts[4], parts[5], parts[6])
	config = open("../log/%s/%s.config" % (db_name, expname), "r")
	summary = open("../log/%s/tablefile" % db_name, 'r')

	config_data = {}

	#print "continue"
	#want to be flexible. Store key-value pairs in dictionary.
	for line in config:
		split = line.split()
		config_data[split[0]] = split[1]
	config.close()

	# most of the following variables are STRINGS
	testbed = config_data["testbed:"]
	interface = config_data["interface:"]
	channel = config_data["channel:"]
	powerlevel = config_data["power_level:"]
	if interface == "802.15.4":
		motetype = config_data["mote:"]
	numnodes = config_data["num_nodes:"]
	numpackets = config_data["num_packets:"]
	ipi = config_data["ipi:"]
	csma = config_data["csma:"]
	txtype = config_data["transmission:"]
	if (txtype == "unicast"):
		ack = config_data["ack:"]
		if (ack == "yes"):
			maxretries = config_data["max_retries:"]
	noise = config_data["noise_sample:"]
	if (noise == "yes"):
		numnoisesamples = config_data["num_samples:"]
		noiseipi = config_data["sample_ipi:"]
	if interface == "802.11":
		double_rate = int(config_data["rate:"])
		rate = str(double_rate / 2.0 if (double_rate & 1) else 2)
		packet_size = config_data["packet_size:"]
	else:
		rate = "n/a"
		packet_size = "n/a"
	# read summary statistics table
	prr = (summary.readline()).split()
	#print prr
	perfect = prr[1]
	good = prr[2]
	intermediate = prr[3]
	bad = prr[4]
	dead = prr[5]

	degree = (summary.readline()).split()
	#print degree
	out = degree[1]
	indegree = degree[2]

	beta = (summary.readline()).split()
	#print beta
	high = beta[1]
	low = beta[2]
	summary.close()

	#open new html to write
	newf = open("../reports/" + db_name + "/report.html", "w")
	
	#factor out common tests between printer-friendly version and html version
	def factor_common(line):
		if line == "%%dbname%%":
			newf.write(db_name)
		elif line == "%%expname%%":
			newf.write(expname)
		elif line == "%%testbed%%":
			newf.write(testbed)
		elif line == "%%date%%":
			newf.write(date)
		elif line == "%%time%%":
			newf.write(time)
		elif line == "%%interface%%":
			newf.write(interface)
		elif line == "%%channel%%":
			newf.write(channel)
		elif line == "%%powerlevel%%":
			newf.write(powerlevel)
		elif line == "%%motetype%%":
			newf.write(motetype)
		elif line == "%%numnodes%%":
			newf.write(numnodes)
		elif line == "%%numpackets%%":
			newf.write(numpackets)
		elif line == "%%ipi%%":
			newf.write("%s ms" % ipi)
		elif line == "%%csma%%":
			newf.write(csma)
		elif line == "%%txtype%%":
			newf.write(txtype)
		elif line == "%%ack%%":
			newf.write(ack)
		elif line == "%%maxretries%%":
			newf.write(maxretries)
		elif line == "%%noise%%":
			newf.write(noise)
		elif line == "%%numnoisesamples%%":
			newf.write(numnoisesamples)
		elif line == "%%noiseipi%%":
			newf.write(noiseipi)
		elif line == "%%perfect%%":
			newf.write(perfect)
		elif line == "%%good%%":
			newf.write(good)
		elif line == "%%intermediate%%":
			newf.write(intermediate)
		elif line == "%%bad%%":
			newf.write(bad)
		elif line == "%%dead%%":
			newf.write(dead)
		elif line == "%%out%%":
			newf.write(out)
		elif line == "%%in%%":
			newf.write(indegree)
		elif line == "%%bursty%%":
			newf.write(high)
		elif line == "%%independent%%":
			newf.write(low)
		elif line == "%%rate%%":
			newf.write("%s Mbps" % rate)
		elif line == "%%packetsize%%":
			newf.write("%s bytes" % packet_size)
		elif line == "%%802_15_4%%":
			if parts[7] == "1":
				newf.write("display:none")
			else:
				newf.write("display:table-row")
		elif line == "%%802_11%%":
			if parts[7] == "1":
				newf.write("display:table-row")
			else:
				newf.write("display:none")
	while 1:
		line = f.readline()
		if not line:
			break
		line = line.replace("\n", "")
		if line.startswith("%%"):
			factor_common(line)
			if interface == "802.15.4":
				if line == "%%prrvscci%%":
					newf.write(
'''<tr>
<td>
<div id="text">
<u>PRR vs. CCI:</u><br>
The Packet Reception Ratio (PRR) distribution plotted against the Chip
Correlation Indicator (CCI). Packet reception ratio is calculated as the ratio of
packets successfully acknowledged to total transmitted packets over a link. The
chip correlation indicator is averaged over all packets successfully received
and the error bars represent 1 standard deviation in CCI values.
<br>
<br>
<img src="plots/prr-cci.png">
</td>
</tr>''')
				elif line == "%%rssivscci%%":
					newf.write(
'''<tr>
<td>
<div id="text">
<u>RSSI vs. CCI:</u><br>
The Received Signal Strength Indicator (RSSI) plotted against the Chip
Correlation Indicator (CCI). Both RSSI and CCI values are averaged over all
packets successfully received over a link. The error bars show 1 standard
deviation of CCI.
<br>
<br>
<img src="plots/rssi-cci.png">
</td>
</tr>''')
				elif line == "%%prrvssnr%%":
					newf.write(
'''<u>PRR vs. RSSI:</u><br>
The Packet Reception Ratio (PRR) distribution plotted against the Received
Signal Strength Indicator (RSSI). Packet reception ratio is calculated as the
ratio of packets successfully acknowledged to total transmitted packets over a
link. RSSI values are averaged over all packets successfully received over a
link.
<br><br><img src="plots/prr-rssi.png">
''');
			#interface is not 802.15.4
			elif line == "%%prrvssnr%%":
				newf.write(
'''<u>PRR vs. SNR:</u><br>
The Packet Reception Ratio (PRR) distribution plotted against the Signal-Noise Ratio (SNR).
Packet reception ratio is calculated as the ratio of packets successfully acknowledged to 
total transmitted packets over a link. SNR values are averaged over all packets successfully 
received over a link.
<br><br><img src="plots/prr-snr.png">
''');
		else:
			if line == "<!-- prr-arr":
				if (txtype == "unicast" and ack == "yes"):
					newf.write("\n")
					uncomment_flag = 1
				else:
					newf.write(line)
					newf.write("\n")
			elif line == "<!-- noisedistr":
				if (noise == "yes"):
					newf.write("\n")
					uncomment_flag = 1
				else:
					newf.write(line)
					newf.write("\n")
			elif line == "<!-- snr-asymm":
				if (noise == "yes"):
					newf.write("\n")
					uncomment_flag = 1
				else:
					newf.write(line)
					newf.write("\n")
			elif line == "<!-- etxasymm":
				if (txtype == "unicast" and ack == "yes"):
					newf.write("\n")
					uncomment_flag = 1
				else:
					newf.write(line)
					newf.write("\n")
			elif line == "<!-- beta":
				if (txtype == "broadcast" or ack == "no"):
					#print "trying to generate beta"
					newf.write("\n")
					uncomment_flag = 1
				else:
					newf.write(line)
					newf.write("\n")
			elif line == "<!-- rho":
				if int(numnodes) > 2:
					newf.write("\n")
					uncomment_flag = 1
				else:
					newf.write(line + "\n")
			elif line == "-->" and uncomment_flag == 1:
				#print("uncomment flag is " + str(uncomment_flag))
				#print("removing line -->")
				newf.write("\n")
				uncomment_flag = 0
			else:
				newf.write(line)
				newf.write("\n")

	f.close()
	newf.close()

#start generating the printer-friendly version

	f = open("printer_friendly.html", "r")
	newf = open("../reports/" + db_name + "/printer.html", "w")
	while 1:
		line = f.readline()
        
		if not line:
			break
		line = line.replace("\n", "")
		if line.startswith("%%"):
			factor_common(line)
			if interface == "802.15.4":
				if line == "%%prrvscci%%":
					newf.write(
'''<tr>
<td>
<div id="text">
<u>PRR vs. CCI:</u><br>
The Packet Reception Ratio (PRR) distribution plotted against the Chip
Correlation Indicator (CCI). Packet reception ratio is calculated as the ratio of
packets successfully acknowledged to total transmitted packets over a link. The
chip correlation indicator is averaged over all packets successfully received
and the error bars represent 1 standard deviation in CCI values.
<br>
<br>
<img src="plots/prr-cci.png">
</td>
</tr>''')
				elif line == "%%rssivscci%%":
					newf.write(
'''<tr>
<td>
<div id="text">
<u>RSSI vs. CCI:</u><br>
The Received Signal Strength Indicator (RSSI) plotted against the Chip
Correlation Indicator (CCI). Both RSSI and CCI values are averaged over all
packets successfully received over a link. The error bars show 1 standard
deviation of CCI.
<br>
<br>
<img src="plots/rssi-cci.png">
</td>
</tr>''')
				elif line == "%%prrvssnr%%":
					newf.write(
'''<u>PRR vs. RSSI:</u><br>
The Packet Reception Ratio (PRR) distribution plotted against the Received
Signal Strength Indicator (RSSI). Packet reception ratio is calculated as the
ratio of packets successfully acknowledged to total transmitted packets over a
link. RSSI values are averaged over all packets successfully received over a
link.
<br><br><img src="plots/prr-rssi.png">
''');
    
            #interface is 802.11, so show SNR instead of RSSI
			elif line == "%%prrvssnr%%":
				newf.write(
'''<u>PRR vs. SNR:</u><br>
The Packet Reception Ratio (PRR) distribution plotted against the Signal-Noise Ratio (SNR).
Packet reception ratio is calculated as the ratio of packets successfully acknowledged to 
total transmitted packets over a link. SNR values are averaged over all packets successfully 
received over a link.
<br><br><img src="plots/prr-snr.png">
''');
		else:
        
			if line == "<!-- prr-arr":
				if (txtype == "unicast" and ack == "yes"):
					newf.write("\n")
					uncomment_flag = 1
				else:
					newf.write(line)
					newf.write("\n")
			elif line == "<!-- noisedistr":
				if (noise == "yes"):
					newf.write("\n")
					uncomment_flag = 1
				else:
					newf.write(line)
					newf.write("\n")
			elif line == "<!-- snr-asymm":
				if (noise == "yes"):
					newf.write("\n")
					uncomment_flag = 1
				else:
					newf.write(line)
					newf.write("\n")
			elif line == "<!-- etxasymm":
				if (txtype == "unicast" and ack == "yes"):
					newf.write("\n")
					uncomment_flag = 1
				else:
					newf.write(line)
					newf.write("\n")
			elif line == "<!-- beta":
				if (txtype == "broadcast" or ack == "no"):
					#print "trying to generate beta"
					newf.write("\n")
					uncomment_flag = 1
				else:
					newf.write(line)
					newf.write("\n")
			elif line == "<!-- rho":
				if int(numnodes) > 2:
					newf.write("\n")
					uncomment_flag = 1
				else:
					newf.write(line + "\n")
			elif line == "-->" and uncomment_flag == 1:
				#print("uncomment flag is " + str(uncomment_flag))
				#print("removing line -->")
				newf.write("\n")
				uncomment_flag = 0
			else:
				newf.write(line)
				newf.write("\n")

	f.close()
	newf.close()
	
	#write the latex table
	mock_table = open("mock_latex_table.tex", "r")
	table_file = open("../log/%s/latex_table.tex" % db_name, "w")
	for line in mock_table:
		if line.find("%%out%%") != -1:
			line = line.replace("%%out%%", out, 1)
		elif line.find("%%in%%") != -1:
			line = line.replace("%%in%%", indegree, 1)
		elif line.find("%%perfect%%") != -1:
			line = line.replace("%%perfect%%", perfect, 1)
		elif line.find("%%middle%%") != -1:
			line = line.replace("%%middle%%", intermediate, 1)
		elif line.find("%%dead%%") != -1:
			line = line.replace("%%dead%%", dead, 1)
		elif line.find("%%bursty%%") != -1:
			line = line.replace("%%bursty%%", high, 1)
		elif line.find("%%independent%%") != -1:
			line = line.replace("%%independent%%", low, 1)
		table_file.write(line)
	mock_table.close()
	table_file.close()

#gen_report("maria_Sep_26_2008_09_59_46")
