package {
	import flare.animate.TransitionEvent;
	import flare.animate.Transitioner;
	import flare.data.DataSet;
	import flare.data.DataSource;
	import flare.display.TextSprite;
	import flare.physics.DragForce;
	import flare.physics.IForce;
	import flare.physics.NBodyForce;
	import flare.scale.ScaleType;
	import flare.util.Colors;
	import flare.util.Property;
	import flare.util.Shapes;
	import flare.util.Strings;
	import flare.util.palette.ColorPalette;
	import flare.vis.Visualization;
	import flare.vis.controls.DragControl;
	import flare.vis.controls.TooltipControl;
	import flare.vis.data.Data;
	import flare.vis.data.EdgeSprite;
	import flare.vis.data.NodeSprite;
	import flare.vis.data.render.ArrowType;
	import flare.vis.events.TooltipEvent;
	import flare.vis.operator.encoder.ColorEncoder;
	import flare.vis.operator.label.Labeler;
	import flare.vis.operator.layout.ForceDirectedLayout;
	import flare.vis.operator.layout.Layout;
	
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.events.TimerEvent;
	import flash.geom.Rectangle;
	import flash.net.URLLoader;
	import flash.utils.Timer;
[SWF(width="900", height="700", backgroundColor="#ffffff", frameRate="30")]
	public class Animation extends Sprite
	{
		private static const EXPECTED:String = "http://127.0.0.1:8080/report_progress/expected.txt";
		private static const TEMP_NODE:String = "http://127.0.0.1:8080/report_progress/temp_node.txt";
		private static const DATA:String = "http://127.0.0.1:8080/report_progress/data.txt";
		//private static const DATA:String = "/home/jchu/src/papers/swat-code/ui/report_progress/data_.txt";
		//private static const TEMP_NODE:String = "/home/jchu/src/papers/swat-code/ui/report_progress/temp_node_.txt";
		//private static const EXPECTED:String = "/home/jchu/src/papers/swat-code/ui/report_progress/expected_.txt";
		/* use this sprite to send debug information to output */
		private static var debug:TextSprite;
		private static const spec_height:int = 700;
		private static const spec_width:int = 700;
		private var node_label:Labeler;
		private var edge_label:Labeler;
		private var data:Data;
		private var refresh:Timer;
		private var vis:Visualization;
		/* Nodes that were inserted in the visualization 
		node properties:
		data.label - the label the node should have
		data.value - the total number of packets received at that node
		data.table_data - the text sprite representing the node packet reception ratio
		*/
		private var hash_nodes:Object = {};
		/* Edges that were inserted in the visualization 
		edge properties:
		data.current - was the edge added in this round?
		data.label - total packets received on that link / total packets sent on that link
		*/
		private var hash_edges:Object = {};
		private var previous_round:String = null;
		private var total:int;
		/* common properties of the nodes */
		private var nodes_prop:Object = {
			shape: Shapes.CIRCLE,
			fillColor: 0xffffb6d3,
			lineColor: 0xff00fff0,
			lineWidth: 1,
			visible: true
		}
		/* gradient to use on the links */
		private var palette:ColorPalette;
		/* predicted, then actual number of nodes */
		private var num_nodes:uint;
		/* encodes the color the edges will take */
		private var color_enc:ColorEncoder;
		/* the selected node (will highlight the node on the left). This is included to allow partial "scrolling"
		of the list. */
		private var node_selected:NodeSprite = null;
		/* edges that should be removed at the end of the transition */
		private var edges_to_remove:Object = {};
		/* variables below deal with initialization */
		private var positions:Array = new Array();
		/* calculates the radius of the circle which the nodes should trace out */
		private function calculate_radius(rad1:Number, rad2:Number, d:int):Number 
		{
			var xDistance:Number = Math.cos(rad2) - Math.cos(rad1);
			var yDistance:Number = Math.sin(rad2) - Math.sin(rad1);
			var temp:Number = Math.sqrt(xDistance * xDistance + yDistance * yDistance);
			return d / temp;
		}
		public function Animation()
		{
			super();
			/* initialize data */
			total = 0;
			
			var colorArray:Array = new Array();
			colorArray.push(Colors.rgba(255, 0, 0));
			for (var i:int = 31; i <= 255; i += 32)
				colorArray.push(Colors.rgba(255, i, 0));
			for (i = 255; i >= 31; i -= 32)
				colorArray.push(Colors.rgba(i, 255, 0));
			colorArray.push(Colors.rgba(0, 255, 0));
			palette = new ColorPalette(colorArray);
			color_enc = new ColorEncoder("data.decimal", Data.EDGES, "lineColor", ScaleType.LINEAR, palette); 
			color_enc.filter = function (edge:EdgeSprite):Boolean
		    {
				return edge.data.current;
			};
			refresh = new Timer(3000);
			init_debug();
			addChild(debug);
			GradientEdgeRender.palette = palette;
			GradientEdgeRender.debug = debug;
			data = new Data();
			setup_main_vis();
			/* get expected number of nodes, so we can make preliminary positions for them */
			var expected:DataSource = new DataSource(EXPECTED, "tab");
			var expected_load:URLLoader = expected.load();
			expected_load.addEventListener(Event.COMPLETE, function (evt:Event):void {
				var rows:Array = ((evt.target as URLLoader).data as DataSet).nodes.data;
				num_nodes = int(rows[0].header); //if it returns 0, we are still okay
				/* get the node IDs so we know how to label the nodes */
				var id_file:DataSource = new DataSource(TEMP_NODE, "tab");
				var load_id:URLLoader = id_file.load();
				if (isNaN(Number(rows[0].header)))
				{
					load_id.addEventListener(Event.COMPLETE, function (evt:Event):void {
						var ds:DataSet = load_id.data as DataSet;
						num_nodes = ds.nodes.data.length;
						first_invocation(ds);
						start_link_animation();
					});
				} else
				{
					load_id.addEventListener(Event.COMPLETE, function (evt:Event):void {
						first_invocation(load_id.data as DataSet);
						refresh.addEventListener(TimerEvent.TIMER, initialize_func);
						refresh.start();
					});
				}
			});
		}
		private function first_invocation(ds:DataSet):void
		{
			generate_positions();
			add_new_nodes_to_vis(ds.nodes.data);
		}
		private function initialize_func(evt:Event):void
		{
			var expected:DataSource = new DataSource(EXPECTED, "tab");
			var expected_load:URLLoader = expected.load();
			expected_load.addEventListener(Event.COMPLETE, function (evt:Event):void {
				/* get the node IDs so we know how to label the nodes */
				var id_file:DataSource = new DataSource(TEMP_NODE, "tab");
				var load_id:URLLoader = id_file.load();
				var rows:Array = ((evt.target as URLLoader).data as DataSet).nodes.data
				if (isNaN(Number(rows[0].header)))
				{
					load_id.addEventListener(Event.COMPLETE, function (evt:Event):void {
						var ds:DataSet = load_id.data as DataSet;
						add_new_nodes_to_vis(ds.nodes.data);
						refresh.stop();
						refresh.removeEventListener(TimerEvent.TIMER, initialize_func);
						start_link_animation();
					});
				} else
				{
					load_id.addEventListener(Event.COMPLETE, function (evt:Event):void {
						var ds:DataSet = load_id.data as DataSet;
						add_new_nodes_to_vis(ds.nodes.data);
					});
				}
			});
		}
		private static function init_debug():void
		{
			debug = new TextSprite("");
			debug.x = 250;
			debug.y = 250;
		}
		/* creates 2 dummy nodes so that the edge color directly correlates with the reception ratio.
		Without them, the color encoder bases the color of each link on the reception ratio compared to other links.
		The 2 edges are created with the max and min value used by the color encoder of 0.0 and 1.0 */
		private static function add_dummy(data:Data):void
		{
			var dummy_start:NodeSprite = data.addNode({table_data: null});
    		var dummy_end:NodeSprite = data.addNode({table_data: null});
    		var start_end:EdgeSprite = data.addEdgeFor(dummy_start, dummy_end, true, {current: true, decimal: 0.0});
    		var end_start:EdgeSprite = data.addEdgeFor(dummy_end, dummy_start, true, {current: true, decimal: 1.0});
    		var visible:Property = Property.$("visible");
    		visible.setValue(dummy_start, false);
    		visible.setValue(dummy_end, false);
    		visible.setValue(start_end, false);
    		visible.setValue(end_start, false);
    		
    		/* visualization breaks if the dummy nodes are put on top of each other */
    		var x_pos:Property = Property.$("x");
    		x_pos.setValue(dummy_start, 200);
    		x_pos.setValue(dummy_end, 250);
    		var y_pos:Property = Property.$("y");
    		y_pos.setValue(dummy_start, 200);
    		y_pos.setValue(dummy_end, 250);
		}
		private function setup_main_vis():void
		{
			/* switch the animation to a built-in one (with a physics engine) */
			vis = new Visualization(data);
			vis.bounds = new Rectangle(0,0,spec_width,spec_height);
			vis.x = 0;
			vis.y = 0;
			var layout:ForceDirectedLayout = setup_layout();
			var force:IForce;
			for (var i:uint = 0; (force = layout.simulation.getForceAt(i)) != null; i++)
			{
				if (force is NBodyForce)
				{
					var NBody:NBodyForce = force as NBodyForce;
					NBody.maxDistance = 15; //objects keep moving away until this distance
					NBody.gravitation = -5; //negative means objects repel each other
					NBody.minDistance = 0;
				} else if (force is DragForce)
				{
					var drag:DragForce = force as DragForce;
					drag.drag = .2; //how fast do objects slow down
				}
			}
			vis.operators.add(layout);
			vis.controls.add(new DragControl(NodeSprite));
			vis.continuousUpdates = true;
			vis.operators.add(color_enc);
			node_label = new Labeler("data.label", Data.NODES, null, NodeSprite, Labeler.LAYER);
			node_label.filter = function(node:NodeSprite):Boolean
			{
				return node.data.table_data != null;
			};
			node_label.horizontalAnchor = 15;
			node_label.verticalAnchor = 15;
			vis.operators.add(node_label);
			vis.controls.add(new TooltipControl(EdgeSprite, null, function (evt:TooltipEvent):void {
				var curr_edge:EdgeSprite = evt.edge;
				var target_str:String = curr_edge.target.data.label;
				var source_str:String = curr_edge.source.data.label;
				/* find the reverse link */
				var reverse:String = (curr_edge.directed ? "" : Strings.format("\nNode {0} --> Node {1}: {2}", 
					target_str, source_str, curr_edge.data.back_label));
				(evt.tooltip as TextSprite).text = Strings.format("Node {0} --> Node {1}: {2}{3}", 
					source_str, target_str, curr_edge.data.label, reverse);
			}));
			edge_label = new Labeler("data.label", Data.EDGES, null, EdgeSprite, Labeler.CHILD);
			edge_label.filter = function (edge:EdgeSprite):Boolean
		    {
				return edge.data.current;
			};
			vis.operators.add(edge_label);
			addChild(vis);
			vis.update();
		}
		private function start_link_animation():void
		{
			num_nodes = data.nodes.length;
			add_dummy(data);
			generate_table();
			refresh.addEventListener(TimerEvent.TIMER, function(evt:TimerEvent):void {
				var file_to_load:DataSource = new DataSource(DATA, "tab");
				var loader:URLLoader = file_to_load.load();
				/* load data */
				loader.addEventListener(Event.COMPLETE, load_complete);
			});
			/* start timer to update the visualization */
			refresh.start();
		}
		private function generate_table():void
		{
			var sprite:Sprite = new Sprite();
            sprite.scrollRect = new Rectangle(0, 0, 200, spec_height);
            sprite.x = spec_width;
            sprite.y = 0;
            var title:TextSprite = new TextSprite("Node ID \t\tReception ratio");
            sprite.addChild(title);
            var i:int = 1;
            for (var prop:String in hash_nodes)
            {
            	var node:NodeSprite = hash_nodes[prop] as NodeSprite;
            	var text_node:TextSprite = node.data.table_data as TextSprite;
            	text_node.text = prop + "\t\t\t" + node.data.value + "/" + String(total);
            	text_node.x = 0;
            	text_node.y = i * 20;
            	sprite.addChild(node.data.table_data);
            	i++;
            }
			sprite.graphics.lineStyle(1);
			sprite.graphics.beginFill(0xffffff);
			sprite.graphics.drawRect(0, 0, 199, i * 20 + 1);
			sprite.graphics.endFill();
			addChild(sprite);
		}
		private function load_complete(evt:Event):void
		{
			var rows:Array = ((evt.target as URLLoader).data as DataSet).nodes.data;
		    var transmitter:int = rows[0].id;
		    var receiver:int = rows[0].received;
		    var total_round:int = rows[1].id;
			if (total_round == 0)
				return;
		    total = rows[1].received;
		    var current_round:String = String(transmitter);
		    var prop:String;
		    /* has a new round started? */
		    if (previous_round != current_round)
		    {
		    	previous_round = current_round;
		    	/* downgrade all edges down to regular ones */
		    	for (prop in hash_edges)
		    	{
		    		var edge:EdgeSprite = hash_edges[prop] as EdgeSprite;
		    		if (edge.data.current)
		    		{
		    			/* removing, and then adding the edge back, allows us to not change the cache attribute */
						data.removeEdge(edge);
						edge = data.addEdgeFor(edge.source, edge.target, edge.directed, edge.data);
						edge.data.current = false;
						edge.renderer = new GradientEdgeRender();
		    			edge.lineWidth = 3;
		    			edge.arrowType = ArrowType.TRIANGLE;
		    			hash_edges[prop] = edge;
		    		}
		    	}
		    }
		    /* grab data about the number of packets received at each node.
		     This computation assumes that the source has already sent packets to nodes before the receiver 
		     in the nodes file */
		    var last_node:int = num_nodes + 1;
		    var first_node:int = 2;
		    /* broadcast is like unicast except with the receiver id of the last node in the list */
		    if (receiver == -1)
		    	receiver = rows[last_node].id;
		    /* set the number of packets received at each node */
		    for (var i:int = last_node; i >= first_node; i--)
		    {
		    	hash_nodes[rows[i].id].data.value = rows[i].received;
		    }
		    /* set the maximum possible number of packets that could be received at each node */
		    for (i = first_node; rows[i].id != receiver && rows[i].id != transmitter; i++)
		    	hash_nodes[rows[i].id].data.total = total;
		    /* receiver before transmitter? */
		    var before:Boolean = rows[i].id != transmitter;
		    if (before)
		    {
		    	hash_nodes[rows[i].id].data.total = total;
		    	var lower_total:int = total - total_round;
		    	for (i++; rows[i].id != transmitter; i++)
		    		hash_nodes[rows[i].id].data.total = lower_total;
		    }
		    /* transmitter should have received <i>total</i> packets */
		    hash_nodes[rows[i].id].data.total = total;
		    if (!before && i < last_node)
		    {
		    	var higher_total:int = total + total_round;
		    	for (i++; rows[i].id != receiver; i++)
		    		hash_nodes[rows[i].id].data.total = higher_total;
		    	hash_nodes[rows[i].id].data.total = higher_total;
		    }
		    for (i++; i <= last_node; i++)
		    	hash_nodes[rows[i].id].data.total = total;
			
			/* initialize transitioner that causes edges to appear slowly */
			var appear:Transitioner = new Transitioner(2);
			appear.optimize = true;
		    /* add to hash_edges or update the edge stored in hash_edges */
		    for (i = last_node + 1; i < rows.length; i++)
		    {
		    	var edge_str:String = transmitter + " " + rows[i].id;
		    	var label_str:String = rows[i].received + "/" + total_round;
		    	hash_nodes[rows[i].id].data.value += rows[i].received;
		    	var curr_edge:EdgeSprite, prev_edge:EdgeSprite;
	    		
		    	if (hash_edges[edge_str] != undefined)
		    	{
		    		prev_edge = hash_edges[edge_str] as EdgeSprite;
		    		/* don't do anything if nothing changed */
		    		if (prev_edge.data.label == label_str)
		    			continue;
		    		/* remove Edge, then put back edge to get around caching */
		    		data.removeEdge(prev_edge);
		    		curr_edge = data.addEdgeFor(hash_nodes[transmitter], hash_nodes[rows[i].id], prev_edge.directed, prev_edge.data);
		    		curr_edge.data.label = label_str;
		    		curr_edge.data.decimal = 1.0 * rows[i].received / total_round;
		    	} else
		    	{
		    	/* edge to insert (always insert to get around the caching attribute) */
	    			curr_edge = data.addEdgeFor(hash_nodes[transmitter], hash_nodes[rows[i].id], true, 
	    				{label: label_str, current: true, decimal: 1.0 * rows[i].received / total_round});
	    			/* set up transitions only if this edge has not been inserted */
	    			curr_edge.alpha = 0.0;
	    			appear.$(curr_edge).alpha = 1.0;
	    		}
			    var reverse:String = rows[i].id + " " + transmitter;
		    	/* if the reverse edge is present, combine both edges.
		    		Assumes that the reverse link will not have any more packets sent over it. */
			    if (hash_edges[reverse] != undefined)
			    {
			    	prev_edge = hash_edges[reverse] as EdgeSprite;
			    	curr_edge.directed = false;
			    	curr_edge.data.back_decimal = prev_edge.data.decimal;
			    	curr_edge.data.back_label = prev_edge.data.label;
			    	/* if an edge is going to be covered, transition that edge out */
			    	edges_to_remove[reverse] = prev_edge;
			    	appear.$(prev_edge).alpha = 0.0;
			    	delete hash_edges[reverse];
			    }
	    		curr_edge.arrowType = ArrowType.LINES;
	    		curr_edge.lineWidth = 4;
				curr_edge.addEventListener(Event.RENDER, function (evt:Event):void {
					var edge:EdgeSprite = evt.target as EdgeSprite;
					edge.props.label.x = (edge.source.x + edge.target.x) / 2;
					edge.props.label.y = (edge.source.y + edge.target.y) / 2;
				});
				hash_edges[edge_str] = curr_edge;
		    }
		    /* we may need to change the highlighting rectangle if the packet receptions grow too large */
		    for (prop in hash_nodes)
		    {
            	var node:NodeSprite = hash_nodes[prop] as NodeSprite;
            	var text_node:TextSprite = node.data.table_data as TextSprite;
	            text_node.text = prop + "\t\t\t" + node.data.value + "/" + String(node.data.total);
	            if (node == node_selected)
	            	change_selected_node(node, 0, 0xffffffff);
      		}
      		/* remove edges only when the transition ends */
      		appear.addEventListener(TransitionEvent.END, function(evt:Event):void {
      			for (var prop:String in edges_to_remove)
      			{
			    	data.removeEdge(edges_to_remove[prop]);
			    	delete edges_to_remove[prop];
      			}
      		});
      		/* separate the instance updates from the gradual updates */
			vis.update();
			appear.play();
		}
		private function setup_layout():ForceDirectedLayout
		{
			var layout:ForceDirectedLayout = new ForceDirectedLayout();
			layout.defaultSpringTension = 0;
			layout.defaultParticleMass = 1;
			layout.enforceBounds = true;
			layout.layoutType = Layout.CARTESIAN; //allows data in data.nodes to have coordinates
			return layout;
		}
		private function change_selected_node(node_sprite:NodeSprite, background:uint, color:uint):void
		{
			var text_sprite:TextSprite = node_sprite.data.table_data as TextSprite;
    		text_sprite.graphics.beginFill(background);
    		text_sprite.graphics.drawRect(1,0, text_sprite.width - 1, text_sprite.height);
    		text_sprite.graphics.endFill();
    		text_sprite.color = color;
		}
		private function add_new_nodes_to_vis(arr:Array):void
	    {
	    	var expand:Transitioner = new Transitioner(1);
			expand.optimize = true;
	        for (var position:uint = data.nodes.length; position < arr.length; position++) {
	        	var add_node:NodeSprite = data.addNode({label: arr[position].id, value: 0, table_data: new TextSprite(), total: 0});
	        	add_node.buttonMode = true;
	        	add_node.alpha = 0;
	        	add_node.x = positions[position].x;
	        	add_node.y = positions[position].y;
	        	expand.$(add_node).alpha = 1;
	        	add_node.addEventListener(MouseEvent.CLICK, function (evt:Event):void {
	        		var node_sprite:NodeSprite = evt.target as NodeSprite;
	        		/* deselect previous node */
	        		if (node_selected != null)
	        		{
	        			change_selected_node(node_selected, 0xffffff, 0xff000000);
	        		}
	        		/* if previous node selected equals the current node selected, do nothing more */
	        		if (node_sprite == node_selected)
	        			node_selected = null;
	        		else
	        		{
	        			change_selected_node(node_sprite, 0, 0xffffffff);
		        		node_selected = node_sprite;
		        		var text_sprite:TextSprite = node_selected.data.table_data as TextSprite;
		        		var index:uint = int(node_selected.data.label);
		        		var rect:Rectangle = text_sprite.parent.scrollRect;
		        		if (index > 20)
		        			rect.y = int(node_selected.data.label - 20) * 20;
		        		else
		        			rect.y = 0;
		        		text_sprite.parent.scrollRect = rect;
	        		}
	        	});
	            hash_nodes[arr[position].id] = add_node;
	        }
			data.nodes.setProperties(nodes_prop);
			expand.play();
	    }
	    private function generate_positions():void
	    {
	    	var i:int;
	    	/* after some arbitrary number of nodes (like 50), a circle may not be the best layout */
	    	if (num_nodes > 50)
	    	{
	    		var WIDTH:int = 3;
	    		var HEIGHT:int = 2;
	    		/* whether or not to do rows first or columns first. */
	    		const ORDERING:Boolean = true;
	    		/* try to fit a fixed number of rectangles into the visualization such that the rectangles have 
	    		an aspect ratio of 2 and the size is maximized */
	    		var length_each:Number = Math.sqrt(spec_width * spec_height / num_nodes);
	    		/* Number of rectangles that can fit height-wise */
	    		var height:Number = Math.ceil(spec_height / (length_each * HEIGHT)); 
	    		var len_height:Number = spec_height / (height * HEIGHT)
	    		/* Number of rectangles that can fit width-wise */
	    		var width:Number = Math.ceil(spec_width / (length_each * WIDTH));
	    		var len_width:Number = spec_width / (width * WIDTH);
	    		length_each = (len_height > len_width) ? len_width : len_height;
	    		
	    		for (i = 0; i < height; i++)
	    		{
	    			var indexI:int = i * width * WIDTH * HEIGHT;
	    			var outer_rectY:int = i * length_each * HEIGHT;
		    		for (var j:int = 0; j < width; j++)
		    		{
		    			var indexJ:int = indexI + j * WIDTH * HEIGHT;
		    			var outer_rectX:int = j * length_each * WIDTH;
		    			var k:int, l:int, indexK:int, indexL:int, inner_rectY:int, inner_rectX:int;
		    			/* the following code fills the inner rectangle */
		    			if (ORDERING)
			    			for (k = 0; k < HEIGHT; k++)
			    			{
			    				indexK = indexJ + k * WIDTH;
			    				if (indexK >= num_nodes)
			    					break;
			    				inner_rectY = outer_rectY + length_each * k;
			    				for (l = 0; l < WIDTH; l++)
			    				{
			    					indexL = indexK + l;
			    					if (indexL >= num_nodes)
			    						break;
			    					inner_rectX = outer_rectX + length_each * l
			    					/* want to put the node in the middle of the rectangle to deal with squares on the edge */
			    					positions[indexL] = {x: inner_rectX + length_each / 2, y: inner_rectY + length_each / 2};
			    				}
			    			}
			    		else
			    			for (k = 0; k < WIDTH; k++)
			    			{
			    				indexK = indexJ + k * HEIGHT;
			    				if (indexK >= num_nodes)
			    					break;
			    				inner_rectX = outer_rectX + length_each * k;
			    				for (l = 0; l < HEIGHT; l++)
			    				{
			    					indexL = indexK + l;
			    					if (indexL >= num_nodes)
			    						break;
			    					inner_rectY = outer_rectY + length_each * l
			    					/* want to put the node in the middle of the rectangle to deal with squares on the edge */
			    					positions[indexL] = {x: inner_rectX + length_each / 2, y: inner_rectY + length_each / 2};
			    				}
			    			}
		    		}
		    	}
		    }	
	    	else
	    	{
				var each_angle:Number = 2.0 * Math.PI / num_nodes;
				var radius:Number = calculate_radius(0, each_angle, 100);
		    	for (i = 0; i < num_nodes; i++)
	        		positions[i] = {x:spec_width/2 + radius * Math.cos(each_angle * i), 
	        			y:spec_height/2 + radius * Math.sin(each_angle * i)};
	     	}
	    }
	    private function filter_edge(edge:EdgeSprite):Boolean
	    {
			return edge.data.current;
		}
	    private static function alert(str:String):void
	    {
	    	debug.text += str;
	    }
	    private static function set_alert(str:String):void
	    {
	    	debug.text = str;
	    }
	}
}
