package
{
	import flare.display.TextSprite;
	import flare.util.Geometry;
	import flare.util.palette.ColorPalette;
	import flare.vis.data.DataSprite;
	import flare.vis.data.EdgeSprite;
	import flare.vis.data.NodeSprite;
	import flare.vis.data.render.ArrowType;
	import flare.vis.data.render.IRenderer;
	
	import flash.display.GradientType;
	import flash.display.Graphics;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import flash.geom.Rectangle;
/**
 * customized edge renderer that adds bi-directional arrows and gradient color
 */
	public class GradientEdgeRender implements IRenderer
	{
		
		private static const ROOT3:Number = Math.sqrt(3);
		
		// temporary variables
		private var _p:Point = new Point(), _q:Point = new Point();
		public static var debug:TextSprite;
		public static var palette:ColorPalette;
		/** @inheritDoc */
		public function render(d:DataSprite):void
		{
			var e:EdgeSprite = d as EdgeSprite;
			if (e == null) { return; } // TODO: throw exception?
			var g:Graphics = e.graphics;
			// modify end points as needed to accomodate arrow
			if (e.arrowType != ArrowType.NONE)
			{
				var arrow1:Object = {}, arrow2:Object = {};
				// determine arrow head size
				var ah:Number = e.arrowHeight, aw:Number = e.arrowWidth/2;
				if (ah < 0 && aw < 0)
					aw = 1.5 * e.lineWidth;
				if (ah < 0) {
					ah = ROOT3 * aw;
				} else if (aw < 0) {
					aw = ah / ROOT3;
				}
				setArrowAttributes(arrow1, e, ah, e.x1, e.y1, e.x2, e.y2, e.target);
				if (!e.directed)
					setArrowAttributes(arrow2, e, ah, e.x2, e.y2, e.x1, e.y1, e.source);
			}

			// draw the edge
			g.clear(); // clear it out
			setLineStyle(e, g); // set the line style
			if (e.directed)
				g.moveTo(e.x1, e.y1);
			else
				g.moveTo(arrow2.x, arrow2.y);
			g.lineTo(arrow1.x, arrow1.y);
			// draw an arrow
			if (e.arrowType != ArrowType.NONE) {
				// get other arrow points
				draw_arrow(arrow1, e, ah, aw, palette.getColor(e.data.decimal));
				if (!e.directed)
				{
					draw_arrow(arrow2, e, ah, aw, palette.getColor(e.data.back_decimal));
				}
			}
		}
		private function find_angle(x_disp:Number, y_disp:Number):Number
		{	
			var dist:Number = Math.sqrt(x_disp * x_disp + y_disp * y_disp);
			var potential_angle:Number = Math.acos(x_disp / dist);
			var another_angle:Number = Math.asin(y_disp / dist);
			var cos_other:Number = 2 * Math.PI - potential_angle;
			var sin_other:Number = Math.PI - another_angle;
			if (Math.abs(sin_other - potential_angle) < .00001 || Math.abs(sin_other - cos_other) < .00001)
				return sin_other;
			if (another_angle < 0)
				return 2 * Math.PI + another_angle;
			return another_angle;
		}
		private function setLineStyle(e:EdgeSprite, g:Graphics):void
		{
			var m:Matrix = new Matrix();
			var x_disp:Number = e.target.x - e.source.x, y_disp:Number =  e.target.y - e.source.y;
			var angle:Number = Math.atan(y_disp / x_disp);
			var translate_x:Number = e.source.x, translate_y:Number = e.source.y;
			if (x_disp < 0 && y_disp > 0)
			{
				angle += Math.PI;
				x_disp = -x_disp;
				translate_x = e.target.x;
			} else if (x_disp > 0 && y_disp < 0)
			{
				y_disp = -y_disp;
				translate_y = e.target.y;
			} else if (x_disp < 0 && y_disp < 0)
			{
				angle += Math.PI;
				y_disp = -y_disp;
				x_disp = -x_disp;
				translate_x = e.target.x;
				translate_y = e.target.y;
			}
            m.createGradientBox(x_disp, y_disp, angle, translate_x, translate_y);
			/* begin, middle, end colors to use in the color gradient */
			var end:uint = palette.getColor(e.data.decimal);
			g.lineStyle(e.lineWidth, end);
            if (!e.directed)           
			{
            	var begin:uint = palette.getColor(e.data.back_decimal);
				var middle:uint = palette.getColor((e.data.decimal + e.data.back_decimal) / 2);
				g.lineGradientStyle(GradientType.LINEAR, [begin, middle, end], [1, 1, 1], [0, 127, 255], m);
			}
		}
		
		private function setArrowAttributes(arrow:Object, e:EdgeSprite, ah:Number, x1:Number, y1:Number, x2:Number, y2:Number, t:NodeSprite):void
		{
			var dx:Number, dy:Number, dd:Number;
			// get arrow tip point as intersection of edge with bounding box
			var r:Rectangle = t.getBounds(t.parent);
			if (Geometry.intersectLineRect(x1,y1,x2,y2, r, _p,_q) <= 0)
			{
				_p.x = x2; _p.y = y2;
			}
			
			// get unit vector along arrow line
			dx = _p.x - x1; dy = _p.y - y1;
			dd = Math.sqrt(dx*dx + dy*dy);
			dx /= dd; dy /= dd;
			
			// set final point positions
			dd = e.lineWidth/2;
			// if drawing as lines, offset arrow tip by half the line width
			if (e.arrowType == ArrowType.LINES) {
				_p.x -= dd*dx;
				_p.y -= dd*dy;
				dd += e.lineWidth;
			}
			// offset the anchor point (the end point for the edge connector)
			// so that edge doesn't "overshoot" the arrow head
			dd = ah - dd;
			x2 = _p.x - dd*dx;
			y2 = _p.y - dd*dy;
			arrow.x = x2;
			arrow.y = y2;
			arrow.dx = dx;
			arrow.dy = dy;
			arrow.px = _p.x;
			arrow.py = _p.y;
		}
		private function draw_arrow(arrow:Object, e:EdgeSprite, ah:Number, aw:Number, color:uint):void
		{
			// get other arrow points
			var x1:Number = arrow.px - ah*arrow.dx + aw*arrow.dy;
			var y1:Number = arrow.py - ah*arrow.dy - aw*arrow.dx;
			var x2:Number = arrow.px - ah*arrow.dx - aw*arrow.dy;
			var y2:Number = arrow.py - ah*arrow.dy + aw*arrow.dx;
			var g:Graphics = e.graphics;
			
			if (e.arrowType == ArrowType.TRIANGLE) {
				g.lineStyle();
				g.moveTo(arrow.px, arrow.py);
				g.beginFill(color);
				g.lineTo(x1, y1);
				g.lineTo(x2, y2);
				g.endFill();
			} else if (e.arrowType == ArrowType.LINES) {
				g.moveTo(x1, y1);
				g.lineTo(arrow.px, arrow.py);
				g.lineTo(x2, y2);
			}
		}
	}
}